/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.internal.views;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeViewerListener;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeExpansionEvent;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.team.bugs.model.IGenericModel;
import org.eclipse.team.bugs.model.IProviderListener;


class BugViewListener implements ISelectionChangedListener, IOpenListener, IProviderListener, IDoubleClickListener, ITreeViewerListener, IMenuListener, FocusListener {
    
    private final BugView fView;
    private final TreeManager fTreeManager;
    private final EventDispatcher fDispatcher;
    private final CustomControlUpdater fCustomControlUpdater;
    private final GlobalActionManager fActionManager;

    public BugViewListener(BugView view) {
        fView= view;
        fTreeManager= new TreeManager(view);
        fActionManager= new GlobalActionManager(view);
        fDispatcher= new EventDispatcher(view.getExtensions(), fView.getSelection());
        fCustomControlUpdater= new CustomControlUpdater(view.getCustomControlParent(), view.getExtensions());
    }
    
    public void selectionChanged(SelectionChangedEvent event) {
        /**
         * This updates the current selection in the BugView. Everyone who needs
         * the current selection should use BugView.getSelection(), which returns an 
         * object that always contains the current selection!
         */
        fView.getSelection().setSelection(event.getSelection());

        fDispatcher.selectionChanged(event);
        fActionManager.updateSelection(fView.getSelection());
        fCustomControlUpdater.update(fView.getSelection());
        
        // scroll to the selected element, just in case the sash hid it
        ISelection s = fView.getViewer().getSelection();
        if (!s.isEmpty()) {
            Object o = ((IStructuredSelection)s).getFirstElement();
            fView.getViewer().reveal(o);
        }
    }
    
    public void open(OpenEvent event) {
        fDispatcher.open(event);
    }

    public void elementsAdded(IGenericModel parent, IGenericModel[] children) {
        fDispatcher.elementsAdded(parent, children);
        fTreeManager.elementsAdded(parent, children);
    }

    public void elementsChanged(IGenericModel[] elements) {
        fDispatcher.elementsChanged(elements);
        fTreeManager.elementsChanged(elements);
    }

    public void elementsRemoved(IGenericModel parent, IGenericModel[] children) {
        fDispatcher.elementsRemoved(parent, children);
        fTreeManager.elementsRemoved(parent, children);
    }

    public void doubleClick(DoubleClickEvent event) {
        fDispatcher.doubleClick(event);
    }

    public void treeCollapsed(TreeExpansionEvent event) {
        fDispatcher.treeCollapsed(event);
    }

    public void treeExpanded(TreeExpansionEvent event) {
        fDispatcher.treeExpanded(event);
    }

    public void menuAboutToShow(IMenuManager menu) {
        fActionManager.fillContextMenu(menu);
    }

    public void focusGained(FocusEvent e) {
        fDispatcher.focusGained(e);
        fActionManager.updateEnablements();
    }

    public void focusLost(FocusEvent e) {
        fDispatcher.focusLost(e);
    }
}