/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.internal.views;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.team.bugs.extension.BugViewExtension;
import org.eclipse.team.bugs.internal.actions.AddProviderAction;
import org.eclipse.team.bugs.internal.actions.DeleteProviderAction;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;


class GlobalActionManager extends KeyAdapter {
    
    private final BugView fView;
    private final IActionBars fBars;
    
    private final Action fAddProviderAction;
    private final DeleteProviderAction fDeleteAction;

    private final IWorkbenchAction fGlobalCutAction;
    private final IWorkbenchAction fGlobalCopyAction;
    private final IWorkbenchAction fGlobalPasteAction;
    private final IWorkbenchAction fGlobalRefreshAction;
    private final IWorkbenchAction fGlobalDeleteAction;
    private final IWorkbenchAction fGlobalMoveAction;
    private final IWorkbenchAction fGlobalRenameAction;
    
    private final Map fGlobalActions;

    private BugModelSelection fSelection;

    public GlobalActionManager(BugView view) {
        
        fView= view;
        fBars= view.getViewSite().getActionBars();
        fSelection= new BugModelSelection();
        
        final IWorkbenchPartSite site= fView.getSite();
        final IWorkbenchWindow window= site.getWorkbenchWindow();
        
        /**
         * Create actions.
         */
        fAddProviderAction= new AddProviderAction(site);
        fDeleteAction= new DeleteProviderAction(site);
        
        fGlobalCutAction= ActionFactory.CUT.create(window);
        fGlobalCopyAction= ActionFactory.COPY.create(window);
        fGlobalPasteAction= ActionFactory.PASTE.create(window);
        fGlobalMoveAction= ActionFactory.MOVE.create(window);
        fGlobalDeleteAction= ActionFactory.DELETE.create(window);
        fGlobalRefreshAction= ActionFactory.REFRESH.create(window);
        fGlobalRenameAction= ActionFactory.RENAME.create(window);

        /**
         * Configure global actions.
         */
        fGlobalActions= new HashMap();
        fGlobalActions.put(ActionFactory.DELETE.getId(), fDeleteAction);
        
        
        final IActionBars bars= view.getViewSite().getActionBars();
        bars.updateActionBars();
        
        /**
         * Register as listener for DELETE events.
         */
        view.getViewer().getControl().addKeyListener(this);
    }
    
    public void updateSelection(BugModelSelection selection) {
        
        fSelection= selection;
        
        final Map actions= new HashMap(fGlobalActions);
        
        if (fSelection.getSelectedProviders().size() == 1) {
            final IBugProvider provider= fSelection.getSelectedProvider();
            final Map specificActions= fView.getExtensions().getProxy(provider).getGlobalActions();
            for (final Iterator iter = specificActions.keySet().iterator(); iter.hasNext();) {
                final String key= (String)iter.next();
                actions.put(key, specificActions.get(key));
            }
            if (fSelection.providersOnly()) {
                actions.put(ActionFactory.DELETE.getId(), fDeleteAction);
            }
        }
        setGlobalActions(actions);
        updateEnablements();
    }
    
    public void updateEnablements() {
        updateGenericEnablements();
        final IBugProvider provider= fSelection.getSelectedProvider();
        if (provider != null) {
            final BugViewExtension extension= fView.getExtensions().getExtension(provider);
            extension.updateActionEnablements(fSelection.getSelection());
        }
    }
    
    private void updateGenericEnablements() {
        fDeleteAction.setEnablement(fSelection);
    }

    private void setGlobalActions(Map globalActions) {
        fBars.clearGlobalActionHandlers();
        for (Iterator iter = globalActions.keySet().iterator(); iter.hasNext();) {
            final String key = (String) iter.next();
            fBars.setGlobalActionHandler(key, (IAction)globalActions.get(key));
        }
        fBars.updateActionBars();
    }
    
    public void fillContextMenu(IMenuManager menu) {
        
        final MenuManager subMenu= new MenuManager("Add", "add");
        subMenu.add(fAddProviderAction);
        subMenu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
        
        menu.add(subMenu);
        menu.add(new Separator());
        menu.add(fGlobalCutAction);
        menu.add(fGlobalCopyAction);
        menu.add(fGlobalPasteAction);
        menu.add(new Separator());
        menu.add(fGlobalMoveAction);
        menu.add(fGlobalRenameAction);
        menu.add(fGlobalDeleteAction);
        menu.add(fGlobalRefreshAction);
        menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
        
        final IBugProvider provider= fSelection.getSelectedProvider();
        if (provider != null) {
            final BugViewExtension extension= fView.getExtensions().getExtension(provider);
            extension.fillContextMenu(fSelection.getSelection(), menu);
        }
    }

    public void keyPressed(KeyEvent event) {
        if (event.character == SWT.DEL && event.stateMask == 0 && fGlobalDeleteAction.isEnabled()) {
            fGlobalDeleteAction.run();
        }
    }
}
