/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.internal.views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.TreeListener;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.team.bugs.extension.IViewerProxy;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugs.model.IGenericModel;
import org.eclipse.team.bugs.model.IProviderListener;
import org.eclipse.ui.IWorkbenchPart;

/**
 * 
 */
public class ViewerProxy implements IViewerProxy, IProviderListener, ISelectionChangedListener, IOpenListener, FocusListener {
    
    private final BugView fView;
    private final IBugProvider fProvider;
    
    private final Collection fSelectionChangedListeners;
    private final Collection fProviderListeners;
    private final Collection fFocusListeners;
    private final Collection fOpenListeners;
    private final Map fGlobalActions;
    
    private final ArrayList fTreeListeners;
    private final ArrayList fDoubleClickListeners;
        
    public ViewerProxy(BugView view, IBugProvider provider) {
        super();
        fView= view;
        fProvider= provider;
        
        fSelectionChangedListeners= new ArrayList();
        fProviderListeners= new ArrayList();
        fOpenListeners= new ArrayList();
        fGlobalActions= new TreeMap();
        fTreeListeners= new ArrayList();
        fDoubleClickListeners= new ArrayList();
        fFocusListeners= new ArrayList();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#addFocusListener(org.eclipse.swt.events.FocusListener)
     */
    public void addFocusListener(FocusListener listener) {
        if (!fFocusListeners.contains(listener))
            fFocusListeners.add(listener);
    }
    
    public void addSelectionChangedListener(ISelectionChangedListener listener) {
        if (!fSelectionChangedListeners.contains(listener))
            fSelectionChangedListeners.add(listener);
    }
    
    public void addProviderListener(IProviderListener listener) {
        if (!fProviderListeners.contains(listener))
            fProviderListeners.add(listener);
    }

    public void addOpenListener(IOpenListener listener) {
        if (!fOpenListeners.contains(listener))
            fOpenListeners.add(listener);
    }

    public void addDoubleClickListener(IDoubleClickListener listener) {
        if (!fDoubleClickListeners.contains(listener))
            fDoubleClickListeners.add(listener);
    }

    public void addTreeListener(TreeListener listener) {
        if (!fTreeListeners.contains(listener))
            fTreeListeners.add(listener);
    }

    public void removeFocusListener(FocusListener listener) {
        fFocusListeners.remove(listener);
    }
    
    public void removeSelectionChangedListener(ISelectionChangedListener listener) {
        fSelectionChangedListeners.remove(listener);
    }
    
    public void removeProviderListener(IProviderListener listener) {
        fProviderListeners.remove(listener);
    }
    
    public void removeOpenListener(IOpenListener listener) {
        fOpenListeners.remove(listener);
    }

    public void removeDoubleClickListener(IDoubleClickListener listener) {
        fDoubleClickListeners.remove(listener);
    }

    public void removeTreeListener(TreeListener listener) {
        fTreeListeners.remove(listener);
    }


    public IWorkbenchPart getPart() {
        return fView;
    }

    public void revealCustomControl() {
    }
    
    public void registerGlobalAction(String id, Action action) {
        fGlobalActions.put(id, action);
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IProviderListener#modelAdded(org.eclipse.team.bugs.model.IGenericModel, org.eclipse.team.bugs.model.IGenericModel)
     */
    public void elementsAdded(IGenericModel parent, IGenericModel[] children) {
        for (Iterator iter = fProviderListeners.iterator(); iter.hasNext();)
            ((IProviderListener) iter.next()).elementsAdded(parent, children);
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IProviderListener#modelChanged(org.eclipse.team.bugs.model.IGenericModel)
     */
    public void elementsChanged(IGenericModel[] elements) {
        for (Iterator iter = fProviderListeners.iterator(); iter.hasNext();)
            ((IProviderListener) iter.next()).elementsChanged(elements);
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IProviderListener#modelRemoved(org.eclipse.team.bugs.model.IGenericModel, org.eclipse.team.bugs.model.IGenericModel)
     */
    public void elementsRemoved(IGenericModel parent, IGenericModel[] children) {
        for (Iterator iter = fProviderListeners.iterator(); iter.hasNext();)
            ((IProviderListener) iter.next()).elementsRemoved(parent, children);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event) {
        for (Iterator iter = fSelectionChangedListeners.iterator(); iter.hasNext();)
            ((ISelectionChangedListener) iter.next()).selectionChanged(event);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.IOpenListener#open(org.eclipse.jface.viewers.OpenEvent)
     */
    public void open(OpenEvent event) {
        for (Iterator iter = fOpenListeners.iterator(); iter.hasNext();)
            ((IOpenListener) iter.next()).open(event);
    }
    
    public IBugProvider getProvider() {
        return fProvider;
    }
    
    public Map getGlobalActions() {
        return fGlobalActions;
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.views.extension.IViewerProxy#getClipboard()
     */
    public Clipboard getClipboard() {
        return fView.getClipboard();
    }

    /**
     * 
     */
    public void dispose() {
        removeDragSupport();
    }


    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#addDragSupport(int, org.eclipse.swt.dnd.Transfer[], org.eclipse.swt.dnd.DragSourceListener)
     */
    public void addDragSupport(Transfer[] transfers, DragSourceListener listener) {
        fView.getDragManager().addDragSupport(fProvider, transfers, listener);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#removeDragSupport()
     */
    public void removeDragSupport() {
        fView.getDragManager().removeDragSupport(fProvider);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#addDropSupport(int, org.eclipse.swt.dnd.Transfer[], org.eclipse.swt.dnd.DropSourceListener)
     */
    public void addDropSupport(int operations, Transfer[] transfers, DropTargetListener listener) {
        fView.getDropManager().addDropSupport(fProvider, operations, transfers, listener);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#removeDropSupport()
     */
    public void removeDropSupport() {
        fView.getDropManager().removeDropSupport(fProvider);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#getViewer()
     */
    public TreeViewer getViewer() {
        return fView.getViewer();
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#getSelection()
     */
    public IStructuredSelection getSelection() {
        return (IStructuredSelection)fView.getViewer().getSelection();
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.extension.IViewerProxy#setRedraw(boolean)
     */
    public void performUpdate(final Runnable runnable) {
        final Tree tree= fView.getViewer().getTree();
        Display.getCurrent().asyncExec(new Runnable() {
            public void run() {
                tree.setRedraw(false);
                try {
                    runnable.run();                    
                } finally {
                    tree.setRedraw(true);
                }
            }
        });
    }

    public void focusGained(FocusEvent e) {
        for (Iterator iter = fFocusListeners.iterator(); iter.hasNext();) {
            ((FocusListener) iter.next()).focusGained(e);
        }
    }

    public void focusLost(FocusEvent e) {
        for (Iterator iter = fFocusListeners.iterator(); iter.hasNext();) {
            ((FocusListener) iter.next()).focusLost(e);
        }
    }
}