/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugs.internal.views.dnd;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.util.Assert;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.Control;
import org.eclipse.team.bugs.internal.views.BugView;
import org.eclipse.team.bugs.model.IBugProvider;

/**
 * This class listens to drag events on the bug view and dispatches the 
 * event to the plugin implementing the selected provider. Every provider 
 * can register exactly one listener.
 */
public class DragManager implements DragSourceListener {
    
    /**
     * A helper class to store the listeners, along with their supported
     * transfers and operations.
     */
    private static class ListenerInfo {
        
        private Object fListener;
        private Transfer [] fTransfers;
        
        public Object getListener() {
            return fListener;
        }
        
        public Transfer [] getTransferTypes() {
            return fTransfers;
        }
        
        public boolean supports(TransferData transferData) {
            final List transfers= Arrays.asList(fTransfers);
            for (Iterator iter = transfers.iterator(); iter.hasNext();) {
                if (((Transfer)iter.next()).isSupportedType(transferData)) 
                    return true;
            }
            return false;
        }
        
        public ListenerInfo(Object listener, Transfer [] transferTypes) {
            fListener= listener;
            fTransfers= transferTypes;
        }
    }
    
    
    private final BugView fView;
    private final Control fControl;
    
    private DragSource fDragMoveSource;
    
    private final Map fListenerInfos;

    /**
     * The supported transfers. Duplicate entries mean that a transfer is supported by more than one
     * listener.
     */
    private final List fTransfers;
    
    private ListenerInfo fCurrent;
    
    
    public DragManager(BugView view) {
        fView= view;
        fListenerInfos= new HashMap();
        fTransfers= new ArrayList();
        fControl= view.getViewer().getControl();
        fCurrent= null;
        fDragMoveSource= new DragSource(fControl, DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK | DND.DROP_DEFAULT | DND.DROP_TARGET_MOVE);
        fDragMoveSource.addDragListener(this);
    }

    /** 
     * The event is dispatched to the selected provider if the listener registered for this provider
     * matches the type of transfer and operation. Else, nothing is done.
     * @see org.eclipse.swt.dnd.DragSourceListener#dragStart(org.eclipse.swt.dnd.DragSourceEvent)
     */
    public void dragStart(DragSourceEvent event){
        fCurrent= getListenerInfo();
        if (fCurrent == null) {
            event.doit= false;
        } else {
            ((DragSourceListener)fCurrent.getListener()).dragStart(event);
            if (!event.doit)
                fCurrent= null;
        }
    }
    
    /** 
     * The event is dispatched to the selected provider if the listener registered for this provider
     * matches the type of transfer and operation. Else, nothing is done.
     * @see org.eclipse.swt.dnd.DragSourceListener#dragSetData(org.eclipse.swt.dnd.DragSourceEvent)
     */
    public void dragSetData(DragSourceEvent event){
        Assert.isNotNull(fCurrent);
        ((DragSourceListener)fCurrent.getListener()).dragSetData(event);
    }

    /** 
     * The event is dispatched to the selected provider if the listener registered for this provider
     * matches the type of transfer and operation. Else, nothing is done.
     * @see org.eclipse.swt.dnd.DragSourceListener#dragFinished(org.eclipse.swt.dnd.DragSourceEvent)
     */
    public void dragFinished(DragSourceEvent event){
        Assert.isNotNull(fCurrent);
        ((DragSourceListener)fCurrent.getListener()).dragFinished(event);
        fCurrent= null;
    }

    /**
     * Register a <code>DragSourceListener</code> for the given provider. For every provider, one listener
     * can be registered. If this method is called and a listener for the given provider is already registered, 
     * the previous listener is removed and the new one is added.
     * @param provider the provider
     * @param transferTypes the supported types
     * @param listener the listener to be notified
     */
    public void addDragSupport(IBugProvider provider, Transfer[] transferTypes, DragSourceListener listener) {
	    if (fListenerInfos.containsKey(provider))
	        internalRemoveSupport(provider);
        fListenerInfos.put(provider, new ListenerInfo(listener, transferTypes));
        fTransfers.addAll(Arrays.asList(transferTypes));
        updateDragSource();
	}
    
    /**
     * Remove the listener for the given provider.
     * @param provider the provider.
     */
    public void removeDragSupport(IBugProvider provider) {
        internalRemoveSupport(provider);
        updateDragSource();
    }
    
    /**
     * Update the drag source with the set of all supported transfer types.
     */
    private void updateDragSource() {
        final Set set= new HashSet(fTransfers);
        final Transfer [] transfers= new Transfer[set.size()];
        set.toArray(transfers);
        fDragMoveSource.setTransfer(transfers);
    }

    /**
     * A helper method to unregister the listener for a provider. 
     * @param provider the provider
     */
    private void internalRemoveSupport(IBugProvider provider) {
        final ListenerInfo info= (ListenerInfo)fListenerInfos.remove(provider);
        if (info == null)
            return;
        final Transfer [] transfers= info.getTransferTypes();
        for (int i = 0; i < transfers.length; i++) {
            fTransfers.remove(transfers[i]);
        }
    }
    
    private ListenerInfo getListenerInfo() {
        final IBugProvider provider= fView.getSelection().getSelectedProvider();

        if (provider == null)
            return null;
    
        return (ListenerInfo)fListenerInfos.get(provider);
    }
}
