/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.actions;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.Util;
import org.eclipse.team.bugzilla.model.BugzillaContainer;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionDelegate;

/**
 * A superclass for handling selection in action delegates
 * (i.e. action contributed to org.eclipse.ui.popupMenus)
 */
public abstract class BugzillaActionDelegate extends ActionDelegate implements IViewActionDelegate {

    private IStructuredSelection selection;
    private IWorkbenchPart targetPart;

    /* (non-Javadoc)
     * @see org.eclipse.ui.actions.ActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			this.selection = (IStructuredSelection) selection;
			if (action != null) {
				setActionEnablement(action);
			}
		}
    }
    
	/**
	 * Method invoked from <code>selectionChanged(IAction, ISelection)</code> 
	 * to set the enablement status of the action.
	 * 
	 * This method can be overridden by subclasses but should not be invoked by them.
	 */
	protected void setActionEnablement(IAction action) {
		try {
			action.setEnabled(isEnabled());
		} catch (CoreException e) {
			action.setEnabled(false);
			// We should not open a dialog when determining menu enablements so log it instead
			BugzillaPlugin.log(e);
		}
	}
	
	/**
	 * Concrete action enablement code.
	 * Subclasses must implement.
	 * 
	 * @return whether the action is enabled
	 * @throws CoreException if an error occurs during enablement detection
	 */
	abstract protected boolean isEnabled() throws CoreException;
	
	/**
	 * Return the selection.
	 * @return the selection
	 */
    public final IStructuredSelection getSelection() {
        return selection;
    }
    
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
	 */
	public void init(IViewPart view) {
		targetPart = view;
	}
	
	/*
	 * Method declared on IObjectActionDelegate.
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		if(targetPart != null) {
			this.targetPart = targetPart;
		}
	}
	
	/**
	 * Return the target part for the action or <code>null</code>
	 * if it was not set.
	 * @return the target part for the action or <code>null</code>
	 */
    public IWorkbenchPart getTargetPart() {
        return targetPart;
    }
    
    protected BugzillaContainer getSelectedContainer() {
        if (selection == null) return null;
        
        if (!(selection.getFirstElement() instanceof BugzillaContainer))
            return null;

        return (BugzillaContainer)selection.getFirstElement();
    }
    
    /**
     * @param e
     */
    protected void handleError(Throwable e) {
        Util.handleError(getTargetPart(), e);
    }
}
