/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.adapters;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.team.bugzilla.browser.IWebContentAdapter;
import org.eclipse.team.bugzilla.model.BugzillaAttachment;
import org.eclipse.team.bugzilla.model.BugzillaComment;
import org.eclipse.team.bugzilla.model.BugzillaFolder;
import org.eclipse.team.bugzilla.model.BugzillaHistoryFolder;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.BugzillaProvider;
import org.eclipse.team.bugzilla.model.BugzillaQuery;
import org.eclipse.team.bugzilla.model.BugzillaReport;
import org.eclipse.team.bugzilla.persistence.FolderPersistenceAdapter;
import org.eclipse.team.bugzilla.persistence.HistoryPersistenceAdapter;
import org.eclipse.team.bugzilla.persistence.IPersistenceAdapter;
import org.eclipse.team.bugzilla.persistence.ProviderPersistenceAdapter;
import org.eclipse.team.bugzilla.persistence.QueryPersistenceAdapter;
import org.eclipse.team.bugzilla.persistence.ReportPersistenceAdapter;
import org.eclipse.team.bugzilla.view.IPresentableAdapter;
import org.eclipse.ui.IActionFilter;
import org.eclipse.ui.model.IWorkbenchAdapter;

/**
 * Bugzilla adapter factor for IPresentableAdapter.class, IWorkbenchAdapter.class, 
 * IWebContentAdapter.class, IActionFilter.class
 */
public class BugzillaAdapterFactory implements IAdapterFactory {
    
    /*
     * IActionFilter adapter for bugzilla model elements
     */
    private final BugzillaModelAdapter fModelAdapter;
    
    /*
     * Table that maps bugzilla model element classes to adapters for
     * IPresentableAdapter.class, IWorkbenchAdapter.class, IWebContentAdapter.class
     */
    private final Map workbenchAdapters;
    
    /*
     * Table that maps bugzilla model element classes to adapters for
     * IPersitenceadapter
     */
    private static final Map persistenceAdapters;
    
    static {
        persistenceAdapters = new HashMap();
        persistenceAdapters.put(BugzillaProvider.class, new ProviderPersistenceAdapter());
        persistenceAdapters.put(BugzillaReport.class, new ReportPersistenceAdapter());
        persistenceAdapters.put(BugzillaFolder.class, new FolderPersistenceAdapter());
        persistenceAdapters.put(BugzillaQuery.class, new QueryPersistenceAdapter());
        persistenceAdapters.put(BugzillaHistoryFolder.class, new HistoryPersistenceAdapter());
    }
    
    public BugzillaAdapterFactory() {
        workbenchAdapters = new HashMap();
        workbenchAdapters.put(BugzillaProvider.class, new BugzillaProviderAdapter());
        workbenchAdapters.put(BugzillaReport.class, new BugzillaReportAdapter());
        workbenchAdapters.put(BugzillaComment.class, new BugzillaCommentAdapter());
        workbenchAdapters.put(BugzillaAttachment.class, new BugzillaAttachmentAdapter());
        workbenchAdapters.put(BugzillaFolder.class, new BugzillaFolderAdapter());
        workbenchAdapters.put(BugzillaQuery.class, new BugzillaQueryAdapter());
        workbenchAdapters.put(BugzillaHistoryFolder.class, new BugzillaHistoryAdapter());
        
        fModelAdapter = new BugzillaModelAdapter();
    }

    public Object getAdapter(Object adaptableObject, Class adapterType) {

        // Handle IActionFilter first
        if (adapterType == IActionFilter.class) {
            if (adaptableObject instanceof BugzillaModel) 
                return fModelAdapter;
            return null;
        }
        
        // Handle IPersistenceAdapter
        if (adapterType == IPersistenceAdapter.class) {
            return getAdapter(adaptableObject, persistenceAdapters, adapterType);
        }
        
        // Look for the adapter in the adapter lookup table
        return getAdapter(adaptableObject, workbenchAdapters, adapterType);
    }

    private Object getAdapter(Object adaptableObject, Map adapterMap, Class adapterType) {
        // Look for the adapter in the adapter lookup table
        Object adapter = adapterMap.get(adaptableObject.getClass());
        if (adapter == null) {
            // There is no direct adapter but the object may be a sublcass of
            // a class which does have an adapter
            for (Iterator iter = adapterMap.keySet().iterator(); iter.hasNext();) {
                Class classObject = (Class) iter.next();
                if (classObject.isInstance(adaptableObject)) {
                    adapter = adapterMap.get(classObject);
                    break;
                }
            }
        }
        // Return the adapter if it implements the requested interface or class
        if (adapter != null && adapterType.isInstance(adapter)) {
            return adapter;
        }
        return null;
    }

    public Class[] getAdapterList() {
        return new Class [] { IPresentableAdapter.class, IWorkbenchAdapter.class, IWebContentAdapter.class, IActionFilter.class, IPersistenceAdapter.class };
    }
    
    public static Object getPersistenceAdapter(Class realClass) {
        return persistenceAdapters.get(realClass);
    }

}
