/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.datatypes;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.util.ListenerList;
import org.eclipse.team.bugs.extension.BugsException;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.model.BugzillaProvider;
import org.eclipse.team.bugzilla.model.BugzillaQueryOptions;
import org.eclipse.team.bugzilla.model.IRemoteModel;
import org.eclipse.team.bugzilla.operations.RefreshOperation;

/**
 * Caches any data fetched from a bugzilla server
 */
public class DataCache {
    
    protected final BugzillaProvider fProvider; 
    private final Map fReports;
    private final Map fAttachments;
    private final Map fQueries;
    
    private final Set fReportErrors;
    private final Set fAttachmentErrors;
    private final Set fQueryErrors;
    
    private final BugzillaQueryOptions queryOptions;
    
    private ListenerList fListeners = new ListenerList();
    
    public DataCache(BugzillaProvider provider) {
        fProvider = provider;
        
        fReports= Collections.synchronizedMap(new HashMap());
        fAttachments= Collections.synchronizedMap(new HashMap());
        fQueries= Collections.synchronizedMap(new HashMap());
        
        fReportErrors= Collections.synchronizedSet(new HashSet());
        fAttachmentErrors = Collections.synchronizedSet(new HashSet());
        fQueryErrors= Collections.synchronizedSet(new HashSet());
        
        /* TODO: jpound - cache query options */
        queryOptions = new BugzillaQueryOptions(provider.getBaseURL());
    }

    /**
     * Return the cached report data for the given bug id.
     * If no data is cached, <code>null</code> is returned.
     * @param id the bug id
     * @return the cached ReportData or <code>null</code>
     */
    public ReportData getReport(String id) {
        return (ReportData)fReports.get(id);
    }

    /**
     * Return the cached attachment for the given id.
     * If no attachment is cached, <code>null</code> is returned.
     * @param desc the attachment id
     * @return the cached attachment or <code>null</code>
     */
    public AttachmentData getAttachment(AttachmentDescription desc) {
        return (AttachmentData)fAttachments.get(desc);
    }
    
    public void refreshReports(String[] reportIds, IProgressMonitor monitor) throws BugzillaException {
        List errors = new ArrayList();
        for (int i = 0; i < reportIds.length; i++) {
            String id = reportIds[i];
            try {
                ReportData data = fProvider.getOperations().getReportData(monitor, id);
                add(data);
                fReportErrors.remove(id);
            } catch (BugzillaException e) {
                errors.add(e);
                fReportErrors.add(id);
            }
        }
        fireReportDataChanged(reportIds);
        handleErrors(errors);
    }
    
    private void handleErrors(List errors) throws BugzillaException {
        if (errors.size() == 1) {
            throw (BugzillaException)errors.get(0);
        } else if (!errors.isEmpty()) {
            MultiStatus status = new MultiStatus(BugzillaPlugin.ID, 0, "Errors occured during refresh", null);
            for (Iterator iter = errors.iterator(); iter.hasNext();) {
                BugsException e = (BugsException) iter.next();
                status.add(e.getStatus());
                throw new BugzillaException(status);
            }
        }
        
    }

    public void refreshAttachments(AttachmentDescription[] descs, IProgressMonitor monitor) throws BugzillaException {
        List errors = new ArrayList();
        for (int i = 0; i < descs.length; i++) {
            AttachmentDescription desc = descs[i];
            try {
                AttachmentData data = fProvider.getOperations().getAttachment(monitor, desc);
                add(desc, data);
                fAttachmentErrors.remove(desc);
            } catch (BugzillaException e) {
                errors.add(e);
                fAttachmentErrors.add(desc);
            }
        }
        fireAttachmentDataChanged(descs);
        handleErrors(errors);
    }
    
    private void add(AttachmentDescription desc, AttachmentData data) {
        fAttachments.put(desc, data);     
    }

    private void fireAttachmentDataChanged(final AttachmentDescription[] descs) {
        Object[] allListeners = fListeners.getListeners();
        for (int i = 0; i < allListeners.length; i++) {
            final IBugRefreshListener listener = (IBugRefreshListener)allListeners[i];
            Platform.run(new ISafeRunnable() {
                public void handleException(Throwable exception) {
                }
                public void run() throws Exception {
                    listener.attachmentsChanged(fProvider, descs);
                }
            });
        }
    }

    private void fireReportDataChanged(final String[] reportIds) {
        Object[] allListeners = fListeners.getListeners();
        for (int i = 0; i < allListeners.length; i++) {
            final IBugRefreshListener listener = (IBugRefreshListener)allListeners[i];
            Platform.run(new ISafeRunnable() {
                public void handleException(Throwable exception) {
                    // Logged by Core
                }
                public void run() throws Exception {
                    listener.reportDataChange(fProvider, reportIds);
                }
            });
        }
    }

    private void add(ReportData data) {
        fReports.put(data.getString(ReportData.ID), data);
    }
    
	public void addRefreshListener(IBugRefreshListener listener) {
		fListeners.add(listener);
	}

	public void removeRefreshListener(IBugRefreshListener listener) {
		fListeners.remove(listener);
	}

    /**
     * @param datas
     * @param monitor
     */
    public void refreshQuery(QueryURL[] datas, IProgressMonitor monitor) throws BugzillaException {
        List errors = new ArrayList();
        for (int i = 0; i < datas.length; i++) {
            QueryURL url = datas[i];
            try {
                QueryData data = fProvider.getOperations().getQueryData(monitor, url);
                if (data == null) {
                	return;
                }
                add(url, data);
                fQueryErrors.remove(url);
            } catch (BugzillaException e) {
                errors.add(e);
                fQueryErrors.add(url);
            }
        }
        fireQueryChanged(datas);
        handleErrors(errors);
    }

    private void fireQueryChanged(final QueryURL[] datas) {
        Object[] allListeners = fListeners.getListeners();
        for (int i = 0; i < allListeners.length; i++) {
            final IBugRefreshListener listener = (IBugRefreshListener)allListeners[i];
            Platform.run(new ISafeRunnable() {
                public void handleException(Throwable exception) {
                    // Logged by Core
                }
                public void run() throws Exception {
                    listener.queryChange(fProvider, datas);
                }
            });
        }
    }

    private void add(QueryURL url, QueryData data) {
        fQueries.put(url, data);
        ReportData[] datas = data.getReportDatas();
        for (int i = 0; i < datas.length; i++) {
            ReportData report = datas[i];
            if (getReport(report.getString(ReportData.ID)) == null) {
                add(report);
            }
        }
    }

    public QueryData getQueryData(QueryURL queryUrl) {
        return (QueryData)fQueries.get(queryUrl);
    }
    
    public boolean hadErrorsForReport(String id) {
        return fReportErrors.contains(id);
    }
    
    public boolean hadErrorsForQuery(QueryURL url) {
        return fQueryErrors.contains(url);
    }
    
    public boolean hadErrorsForAttachment(AttachmentDescription desc) {
        return fAttachmentErrors.contains(desc);
    }
    
    /**
     * 
     * @return The query options for this bugzilla database.
     */
    public BugzillaQueryOptions getQueryOptions() {
    	return queryOptions;
    }

	/**
	 * 
	 */
	public void refreshQueryOptions() {
		try {
			new RefreshOperation(null, new IRemoteModel[] { getQueryOptions() }).run();
		} catch (InvocationTargetException e) {
			System.err.println("InvocationTargetException");
			//handleError(e);
		} catch (InterruptedException e) {
			System.err.println("InterruptedException");
			// Ignore
		}
	}
}
