/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.datatypes;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.eclipse.team.bugzilla.Util;


/**
 * Wrapper for a bugzilla query URL
 */
public class QueryURL extends BugzillaTransactionURL {
	
	public QueryURL(String baseUrl) {
		super(baseUrl);
		
		script = "buglist.cgi?";
		
		// initiallize default parameter settings
		addTransactionParam("format", "rdf");
		addTransactionParam("search", "Search");
	}
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof QueryURL) {
			return toUrlString().equals(((QueryURL)obj).toUrlString());
		}
		return super.equals(obj);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return toUrlString().hashCode();
	}
	
	// generate the query string (.cgi) from the parameter set
	protected String generateTransactionString() {
		
		if (url.equals("")) {
			System.err.println("BugzillaAction: No URL defined for action");
			return url;
		}
		/* A QueryURL object is being created from an already existing query url */
		int index;
		if ( (index = url.indexOf(script)) >= 0) {
			transUrl = url;
			if (transUrl.endsWith("/")) {
				transUrl = transUrl.substring(0, transUrl.lastIndexOf("/"));
			}
			url = url.substring(0, index);
			return transUrl;
		}
		
		transUrl = url + script;
		Iterator fieldIter = ((Set)params.keySet()).iterator();
		
		try {
			while (fieldIter.hasNext()) {
				String field = (String)fieldIter.next();
				Iterator paramIter = ((List)params.get(field)).iterator();
				
				while(paramIter.hasNext()) {
					String value = URLEncoder.encode((String)paramIter.next(), "UTF-8");
					transUrl += "&" + field + "=" + value;
				}
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		
		return transUrl;
	}
	
	// add the specified 'param' to the list mapped to by 'field'
	public void addTransactionParam(String field, String param) {
		if (param == null) {
			return;
		}

		List parameters = (List)params.get(field);
		
		if(parameters == null) {
			parameters = new LinkedList();
			params.put(field, parameters);
		}
		
		parameters.add(param.trim());
	}
	
	// set param fields. These fields can map to ONLY 1 value.
	public void setFormatType(String type) {
		params.put("format", null);     	 	// clear any previous value
		addTransactionParam("format", type); 	// set the new value (only one value allowed for this type)
	}
	
	public void setSearchType(String type) {
		params.put("search", null);     		// clear any previous value
		addTransactionParam("search", type); 	// set the new value (only one value allowed for this type)
	}
	
	public void setShorDescType(String type) {
		params.put("short_desc_type", null);     		// clear any previous value
		addTransactionParam("short_desc_type", type); 	// set the new value (only one value allowed for this type)
	}
	
	public void setLongDescType(String type) { 
		params.put("long_desc_type", null);     		// clear any previous value
		addTransactionParam("long_desc_type", type); 	// set the new value (only one value allowed for this type)
	}
	
	public String getFormatType() {
		return (String)((List)params.get("format")).get(0);
	}
	
	public String toUrlString() {
		return (transUrl.equals("")) ? generateTransactionString() : transUrl;
	}
	
	/**
	 * Return the url that will execute the query and return XML
	 * @param configuration the configuration which identifies the bugzilla server
	 * @return the URL that executes this query
	 */
	public final String getUrl(BugzillaConfiguration configuration) {
		return getUrlString(configuration);
	}
	
	/**
	 * Build the string that is the URL to perform the query
	 * @param configuration the configuration which identifies the bugzilla server
	 * @return a String that is a valid URL
	 */
	protected String getUrlString(BugzillaConfiguration configuration) {
		String urlString = Util.appendPath(configuration.getBaseURL(), toUrlString());
		return urlString;
	}
	
	/**
	 * Returns the base url of the bugzilla database
	 * @return The base url of the bugzilla database
	 */
	public String getBaseUrl() {
		return url;
	}
	
	/**
	 * Return whether the query is a named query which requires
	 * a login.
	 * @return whether the query is a named query
	 */
	public boolean isNamedQuery() {
		return transUrl.indexOf("cmdtype=runnamed") != -1;
	}
	
	/**
	 * @return login cookie
	 */
	public String getLoginCookie() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * Return an appropriate name for the query
	 * @return an appropriate name for the query
	 */
	public String getName() {
		if (isNamedQuery()) {
			String key = "namedcmd="; //$NON-NLS-1$
			int start = transUrl.indexOf(key);
			if (start != -1) {
				start += key.length();
			}
			String name = transUrl.substring(start);
			try {
				return URLDecoder.decode(name, "UTF-8"); //$NON-NLS-1$
			} catch (UnsupportedEncodingException e) {
				// Ignore
			}
		}
		return transUrl;
	}
}
