package org.eclipse.team.bugzilla.dialogs;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.SortedSet;
import java.util.TreeSet;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.team.bugs.dialogs.StatusDialog;
import org.eclipse.team.bugs.tools.SWTUtils;
import org.eclipse.team.bugzilla.datatypes.QueryURL;
import org.eclipse.team.bugzilla.model.BugzillaQueryOptions;


public class AddQueryDialog extends StatusDialog implements ModifyListener {
	
	private Text summaryField;
	private Text commentField;
	private Text urlField;
	private Text whiteboardField;
	private Text emailField;
	private Text bugNumbers;
	private Text bugVotes;
	private Text bugChangeFrom;
	private Text bugChangeTo;
	private Text bugChangeNewValue;
	
	private Combo summarySearchType;
	private Combo commentSearchType;
	private Combo urlSearchType;
	private Combo keywordsSearchType;
	private Combo keywordsSearchValue;
	private Combo whiteboardSearchType;
	private Combo emailSearchType;
	private Combo bugNumbersSearchType;
	private Combo bugVotesSearchType;
		
	private List productList;
	private List componentList;
	private List versionList;
	private List milestoneList;
	private List bugStatusList;
	private List resolutionList;
	private List severityList;
	private List priorityList;
	private List platformList;
	private List osList;
	private List bugChangeList;
	
	private Composite mainComposite = null;
	private Composite advancedComposite = null;
	
	private Button advancedButton;
	private Button bugOwner;
	private Button bugReporter;
	private Button ccMember;
	private Button bugCommenter;
	
	private boolean advancedView = false;
	private BugzillaQueryOptions options;
	private QueryURL queryUrl;
	
	private static HashMap searchTypes = new HashMap(36);
	private static LinkedList displayableSearchTypes = new LinkedList();
	private static LinkedList displayableKeywordTypes = new LinkedList();
	
	/* 
	 * default window values, the width is not final as it will adapt to user changes.
	 */
	public static int DEFAULT_WIDTH = 790;
	public static final int DEFAULT_FULL_HEIGHT = 600;
	public static final int DEFAULT_HALF_HEIGHT = 370;
		
	/* TODO: jpound - externalize strings */
	static {
		/* Possible search types as defined by bugzilla */
		searchTypes.put("contains all of the words/strings", "allwordssubstr");
		searchTypes.put("contains any of the words/strings", "anywordssubstr");
		searchTypes.put("contains the string", "substring");
		searchTypes.put("contains the string (exact case)","casesubstring");
		searchTypes.put("matches the regexp", "regexp");
		searchTypes.put("doesnt match the regexp", "notregexp");
		/* keyword search type mappings */
		searchTypes.put("Contains all of the keywords", "allwords");
		searchTypes.put("Contains any of the keywords", "allwordssubstr");
		searchTypes.put("Contains none of the keywords   ", "nowords");
		/* email search types */
		searchTypes.put("contains", "substring");
		searchTypes.put("equals", "exact");
		/* bug numering search types */
		searchTypes.put("Only include bugs numbered", "inlcude");
		searchTypes.put("Exclude bugs numbered", "exclude");
		/* bug modification search types */
		searchTypes.put("[Bug creation]", "[Bug creation]");
		searchTypes.put("Alias", "alias");
		searchTypes.put("Assignee", "assigned_to");
		searchTypes.put("URL", "bug_file_loc");
		searchTypes.put("Severity", "bug_severity");
		searchTypes.put("Status", "bug_status");
		searchTypes.put("CC list accessible?", "cclist_accessible");
		searchTypes.put("Component", "component");
		searchTypes.put("Ever confirmed?", "everconfirmed");
		searchTypes.put("Keywords", "keywords");
		searchTypes.put("OS", "op_sys");
		searchTypes.put("Priority", "priority");
		searchTypes.put("Product", "product");
		searchTypes.put("QA Contact", "qa_contact");
		searchTypes.put("Hardware", "rep_platform");
		searchTypes.put("Reporter", "reporter");
		searchTypes.put("Reporter accessible?", "reporter_accessible");
		searchTypes.put("Resolution", "resolution");
		searchTypes.put("Summary", "short_desc");
		searchTypes.put("Whiteboard", "status_whiteboard");
		searchTypes.put("Target Milestone", "target_milestone");
		searchTypes.put("Version", "version");
		searchTypes.put("Votes", "votes");
		
		
		/* displayable strings of search types. These should exactly match the mappings above */
		displayableSearchTypes.add("contains all of the words/strings");
		displayableSearchTypes.add("contains any of the words/strings");
		displayableSearchTypes.add("contains the string");
		displayableSearchTypes.add("contains the string (exact case)");
		displayableSearchTypes.add("matches the regexp");
		displayableSearchTypes.add("doesnt match the regexp");
		
		displayableKeywordTypes.add("Contains all of the keywords");
		displayableKeywordTypes.add("Contains any of the keywords");
		displayableKeywordTypes.add("Contains none of the keywords   ");
	}
	
	public AddQueryDialog(Shell parentShell, BugzillaQueryOptions qo) {
		super(parentShell, "Query " + qo.getUrl(), SWT.DEFAULT, SWT.DEFAULT);
		options = qo;
	}
	
	protected Point getInitialSize() {
        return new Point(DEFAULT_WIDTH, DEFAULT_HALF_HEIGHT);
    }
	
	public QueryURL getQuery() {
		return queryUrl;
	}
	
	public String getQueryString() {
		return queryUrl.toUrlString();
	}
	
	/*
	 * Build a bugzilla query string from the selected options
	 */
	private void buildQueryString() {
		String urlString = options.getBaseUrl();
		queryUrl = new QueryURL(urlString);
		
		addSearchFieldContentsToQuery("short_desc_type", "short_desc", summarySearchType, summaryField);

		//queryUrl.setSearchType("Search");
		addListContentsToQuery("product", productList);
		addListContentsToQuery("component", componentList);
		addListContentsToQuery("version", versionList);
		addListContentsToQuery("target_milestone", milestoneList);
		
		if (!advancedView) {
			/* default bug status */
			queryUrl.addTransactionParam(BugzillaQueryOptions.BUG_STATUS, "NEW");
			queryUrl.addTransactionParam(BugzillaQueryOptions.BUG_STATUS, "ASSIGNED");
			queryUrl.addTransactionParam(BugzillaQueryOptions.BUG_STATUS, "REOPENED");
		}
		else {
			addListContentsToQuery(BugzillaQueryOptions.BUG_STATUS, bugStatusList);
			addListContentsToQuery(BugzillaQueryOptions.SEVERITY, severityList);
			addListContentsToQuery(BugzillaQueryOptions.PRIORITY, priorityList);
			addListContentsToQuery(BugzillaQueryOptions.PLATFORM, platformList);
			addListContentsToQuery(BugzillaQueryOptions.RESOLUTION, resolutionList);
			addListContentsToQuery(BugzillaQueryOptions.OPERATING_SYSTEM, osList);
			
			addSearchFieldContentsToQuery("long_desc_type", 		"long_desc", 		 commentSearchType, 	commentField);
			addSearchFieldContentsToQuery("bug_file_loc_type", 		"bug_file_loc", 	 urlSearchType, 		urlField);
			addSearchFieldContentsToQuery("status_whiteboard_type", "status_whiteboard", whiteboardSearchType,  whiteboardField);
			
			/* keywords */
			String fieldValue = keywordsSearchValue.getText();
			if (!fieldValue.equals("")) {
				String searchType = (String)searchTypes.get(keywordsSearchType.getText());
				queryUrl.addTransactionParam("keywords_type", searchType);
				queryUrl.addTransactionParam("keywords", fieldValue);
			}
			
			/* email address */
			String emailAddy = emailField.getText();
			if (!emailAddy.equals("")) {
				if (bugOwner.getSelection()) {
					queryUrl.addTransactionParam("emailassigned_to1", "1");
				}
				if (bugReporter.getSelection()) {
					queryUrl.addTransactionParam("emailreporter1", "1");
				}
				if (ccMember.getSelection()) {
					queryUrl.addTransactionParam("emailcc1", "1");
				}
				if (bugCommenter.getSelection()) {
					queryUrl.addTransactionParam("emaillongdesc1", "1");
				}
				
				queryUrl.addTransactionParam("emailtype1", (String)searchTypes.get(emailSearchType.getText()));
				queryUrl.addTransactionParam("email1", emailAddy);
			}
			
			/* vote count */
			if (bugVotes.getText().length() > 0) {
				int votes = -1;
				try {
					votes = new Integer(bugVotes.getText()).intValue();
				}
				catch (Exception e) {

				}
				if (votes > -1) {
					queryUrl.addTransactionParam("votes", new String("" + (votes + 1)));
				}
			}
			
			/* bug numbering */
			String bugNums = bugNumbers.getText();
			if (bugNums.length() > 0) {
				if (bugNums.indexOf(",") < 0) {
					bugNums = bugNums.replaceAll(" ", ",");
				}
				queryUrl.addTransactionParam("bugidtype", (String)searchTypes.get(bugNumbersSearchType.getText()));
				queryUrl.addTransactionParam("bug_id", bugNums);
			}
			
			/* change from-to */
			String from = bugChangeFrom.getText();
			if (from.length() > 0 && !from.equals("YYYY-MM-DD")) {
				queryUrl.addTransactionParam("chfieldfrom", from);
			}
			String to = bugChangeTo.getText();
			if (to.length() > 0) {
				queryUrl.addTransactionParam("chfieldto", to);
			}
			
			/* changed field */
			String[] fields = bugChangeList.getSelection();
			if (fields.length > 0) {
				for (int i = 0; i < fields.length; i++) {
					queryUrl.addTransactionParam("chfield", (String)searchTypes.get(fields[i]));
				}
			}
			
			/* new changed value */
			String newValue = bugChangeNewValue.getText();
			if (newValue.length() > 0) {
				queryUrl.addTransactionParam("chfieldvalue", newValue);
			}
		}
	}
	
	private boolean addSearchFieldContentsToQuery(String searchTypeVar, String searchValueVar, Combo combo, Text field) {
		String fieldValue = field.getText();
		if (!fieldValue.equals("")) {
			String searchType = (String)searchTypes.get(combo.getText());
			queryUrl.addTransactionParam(searchTypeVar, searchType);
			queryUrl.addTransactionParam(searchValueVar, fieldValue);
			return true;
		}
		return false;
	}
	
	private void addListContentsToQuery(String var, List list) {
		if (list == null) {
			return;
		}
		
		String[] values = list.getSelection();
		for (int i = 0; i < values.length; i++) {
			queryUrl.addTransactionParam(var, values[i]);
		}
	}
	
	/**
	 * 
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		((GridLayout) parent.getLayout()).numColumns += 2;
		
		// a button to switch to the "advanced" query dialog
		advancedButton = new Button(parent, SWT.PUSH);
		advancedButton.setFont(JFaceResources.getDialogFont());
		advancedButton.setText("Advanced >>");
		advancedButton.setToolTipText("Show advanced query options");
		
		advancedButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				if (advancedView) {
					/* 
					 * if the advancedComposite is not destroyed, room is still allocated for it
					 * (even after the call to advancedComposite.setVisible(false) ) so it is
					 * destroyed each time the user switches back to simple mode from advanced.
					 * The alternative is to set the mainComposite to not grab vertical space,
					 * however this gives less user control over the dialog.
					 */
					//advancedComposite.setVisible(false);
					advancedComposite.dispose();
					advancedComposite = null;
										
					advancedView = false;
					advancedButton.setText("Advanced >>");
					advancedButton.setToolTipText("Show advanced query options");
					
					mainComposite.getShell().setSize(mainComposite.getShell().getSize().x, DEFAULT_HALF_HEIGHT);
				}
				else {
					if (advancedComposite == null) {
						createAdvancedDialogArea(mainComposite);
					}
					advancedComposite.setVisible(true);
										
					advancedView = true;
					advancedButton.setText("Simple <<");
					advancedButton.setToolTipText("Hide advanced query options");
					
					/* FIXME: jpound - shells don't like to be resized after they are open. This is a workaround */
					mainComposite.getShell().setSize(mainComposite.getShell().getSize().x+1, DEFAULT_FULL_HEIGHT);
					mainComposite.getShell().setSize(mainComposite.getShell().getSize().x-1, DEFAULT_FULL_HEIGHT);
				
					//mainComposite.getShell().setSize(mainComposite.getShell().getSize().x, DEFAULT_FULL_HEIGHT);
					//mainComposite.redraw();
				}
				checkComplete();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing
			}
		});
		
		final Shell shell = parent.getShell();
		fOKButton = new Button(parent, SWT.PUSH);
		fOKButton.setText(IDialogConstants.OK_LABEL);
		fOKButton.setFont(JFaceResources.getDialogFont());
        
        fOKButton.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				buildQueryString();
				DEFAULT_WIDTH = shell.getSize().x;
				
				close();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				// do nothing
			}
        });
        
		if (shell != null) {
            shell.setDefaultButton(fOKButton);
        }
        
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        int widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
        Point minSize = fOKButton.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
        data.widthHint = Math.max(widthHint, minSize.x);
        fOKButton.setLayoutData(data);
        advancedButton.setLayoutData(data);
        
        createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
        fOKButton.setEnabled(false);
    }
		
	/* (non-Javadoc)
	 * @see org.eclipse.team.bugs.dialogs.StatusDialog#createMainDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Composite createMainDialogArea(Composite parent) {
		
		mainComposite = new Composite(parent, SWT.NONE);
		mainComposite.setLayout(new GridLayout(1, true));
		mainComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
				
		// the composite to hold the simple query UI options
		final Composite simpleComposite = new Composite(mainComposite, SWT.NONE);
		simpleComposite.setLayout(new GridLayout(1, true));
		simpleComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		// a group to hold the search UI features
		Group searchGroup = new Group(simpleComposite, SWT.SHADOW_NONE);
		searchGroup.setText("Search Terms");
		searchGroup.setLayout(new GridLayout(3, false));
		searchGroup.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		int columns = 4;
		// a group to hold the query options
		Group optionsGroup = new Group(simpleComposite, SWT.SHADOW_NONE);
		optionsGroup.setText("Search Options");
		optionsGroup.setLayout(new GridLayout(columns, true));
		optionsGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		// summary search field
		createLabel(searchGroup, "Summary ");
		summarySearchType = createSearchTypeCombo(searchGroup);
		summaryField = createTextField(searchGroup);
		summaryField.setFocus();

		
		createLabel(optionsGroup, "Product:"); 		// product label
		createLabel(optionsGroup, "Component:");	// component label
		createLabel(optionsGroup, "Version:");		// version label
		createLabel(optionsGroup, "Target:"); 		// target milestone label
		productList = createList(optionsGroup);		// product list
		componentList = createList(optionsGroup);	// component list
		versionList = createList(optionsGroup);		// version list
		milestoneList = createList(optionsGroup);	// target milestone list
		
		if (options.needsRefresh()) {
			while (options.isRefreshing()) {
				
			}
		}
		
		// populate the product and component lists
		Iterator iter = options.productIterator();
		if (!iter.hasNext()) {
			productList.add("No products found");
		}
		
		/* Since "Collections" expects a "List" and the HashMap returns a "Set" of keys,
		 * I use a little extra memory to and do a linear traversal of the Set creating
		 * a sorted structure instead of building a list and using Collections.sort().
		 */
		SortedSet sorted = new TreeSet(new Comparator() {
			/* Case insensitive compare */
			public int compare(Object o1, Object o2) {
				return (((String)o1).toUpperCase()).compareTo(((String)o2).toUpperCase());
			}
		});
		
		while (iter.hasNext()) {
			sorted.add(iter.next());
		}
		
		iter = sorted.iterator();
		while (iter.hasNext()) {
			productList.add((String)iter.next());
		}
		
		setComplete(false);
		
		// when a field is selected, set the assiciated options
		productList.addSelectionListener(new BugzillaListSelectionListener(productList, componentList, BugzillaQueryOptions.COMPONENT_INDEX));
		productList.addSelectionListener(new BugzillaListSelectionListener(productList, versionList, BugzillaQueryOptions.VERSION_INDEX));
		productList.addSelectionListener(new BugzillaListSelectionListener(productList, milestoneList, BugzillaQueryOptions.TARGETMILESTONE_INDEX));
		
		return simpleComposite;
	}
	
	private void createAdvancedDialogArea(Composite parent) {
		/* TODO: jpound - refactor this beast into smaller parts */
		advancedComposite = new Group(parent, SWT.SHADOW_NONE);
		advancedComposite.setLayout(new GridLayout(1, true));
		advancedComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		((Group)advancedComposite).setText("Advanced Options");
		
		TabFolder folder = new TabFolder(advancedComposite, SWT.TOP);
		folder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		/* Bug Status tab */
		TabItem bugProperties = new TabItem(folder, SWT.DEFAULT);
		bugProperties.setText("Bug Properties");
		
		Composite bugPropertiesComposite = new Composite(folder, SWT.NONE);
		GridLayout layout = new GridLayout(0, true);
		bugPropertiesComposite.setLayout(layout);
		bugPropertiesComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		bugProperties.setControl(bugPropertiesComposite);
		
		/* create labels for the list options which have values */
		if (options.getOptions(BugzillaQueryOptions.BUG_STATUS) != null) {
			layout.numColumns++;
			createLabel(bugPropertiesComposite, "Status:");
		}
		
		if (options.getOptions(BugzillaQueryOptions.RESOLUTION) != null) {
			layout.numColumns++;
			createLabel(bugPropertiesComposite, "Resolution:");
		}
		
		if (options.getOptions(BugzillaQueryOptions.PRIORITY) != null) {
			layout.numColumns++;
			createLabel(bugPropertiesComposite, "Priority:");
		}
		
		if (options.getOptions(BugzillaQueryOptions.SEVERITY) != null) {
			layout.numColumns++;
			createLabel(bugPropertiesComposite, "Severity:");
		}
		
		if (options.getOptions(BugzillaQueryOptions.PLATFORM) != null) {
			layout.numColumns++;
			createLabel(bugPropertiesComposite, "Platform:");
		}
		
		if (options.getOptions(BugzillaQueryOptions.OPERATING_SYSTEM) != null) {
			layout.numColumns++;
			createLabel(bugPropertiesComposite, "Operating System:");
		}
		
		/* populate lists for options which have values */
		Iterator items = options.optionsIterator(BugzillaQueryOptions.BUG_STATUS);
		if (items != null) {
			bugStatusList = createList(bugPropertiesComposite);
			
			/* The following can be used instead of the "populateList()" 
			 * and "new int[] {1, 2, 3}" calls below to ensure particular 
			 * options are selected 
			*/
			
			java.util.Set selectMe = new java.util.HashSet();
			selectMe.add("NEW"); selectMe.add("ASSIGNED"); selectMe.add("REOPENED");
			java.util.List selectIndex = new java.util.LinkedList();
			int index = 0;
			while (items != null && items.hasNext()) {
				String item = (String)items.next();
				bugStatusList.add(item);
				if (selectMe.contains(item)) {
					selectIndex.add(new Integer(index));
				}
				index++;
			}
			Iterator iter = selectIndex.iterator();
			int[] selectInts = new int[selectIndex.size()];
			index = 0;
			while (iter.hasNext()) {
				selectInts[index] = ((Integer)iter.next()).intValue();
				index++;
			}
			bugStatusList.select(selectInts);
			/*
			populateList(items, bugStatusList);
			bugStatusList.select(new int[] {1, 2, 3});
			*/
		}
		
		items = options.optionsIterator(BugzillaQueryOptions.RESOLUTION);
		if (items != null) {
			resolutionList = createList(bugPropertiesComposite);
			populateList(items, resolutionList);
		}
		
		items = options.optionsIterator(BugzillaQueryOptions.PRIORITY);
		if (items != null) {
			priorityList = createList(bugPropertiesComposite);
			populateList(items, priorityList);
		}
		
		items = options.optionsIterator(BugzillaQueryOptions.SEVERITY);
		if (items != null) {
			severityList = createList(bugPropertiesComposite);
			populateList(items, severityList);
		}
		
		items = options.optionsIterator(BugzillaQueryOptions.PLATFORM);
		if (items != null) {
			platformList = createList(bugPropertiesComposite);
			populateList(items, platformList);
		}
		
		items = options.optionsIterator(BugzillaQueryOptions.OPERATING_SYSTEM);
		if (items != null) {
			osList = createList(bugPropertiesComposite);
			populateList(items, osList);
		}
		
		/* Search field tab */
		TabItem advancedSearch = new TabItem(folder, SWT.DEFAULT);
		advancedSearch.setText("Search Terms");
		
		Composite advancedSearchComposite = new Composite(folder, SWT.NONE);
		GridLayout advSearchLayout = new GridLayout(3, false);
		advSearchLayout.marginHeight = 15;
		advancedSearchComposite.setLayout(advSearchLayout);
		advancedSearchComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		advancedSearch.setControl(advancedSearchComposite);
		
		// comment search field
		createLabel(advancedSearchComposite, "Comment");
		commentSearchType = createSearchTypeCombo(advancedSearchComposite);
		commentField = createTextField(advancedSearchComposite);
		
		// url search field
		createLabel(advancedSearchComposite, "URL");
		urlSearchType = createSearchTypeCombo(advancedSearchComposite);
		urlField = createTextField(advancedSearchComposite);
		
		// whiteboard serach field
		createLabel(advancedSearchComposite, "Whiteboard");
		whiteboardSearchType = createSearchTypeCombo(advancedSearchComposite);
		whiteboardField = createTextField(advancedSearchComposite);
		
		// keyword search field
		createLabel(advancedSearchComposite, "Keywords");
		keywordsSearchType = new Combo(advancedSearchComposite, SWT.SINGLE | SWT.READ_ONLY);
		
		//keywordSearchType.setLayout(new FillLayout(SWT.HORIZONTAL));
		//keywordSearchType.setLayoutData(new GridData(SWT.DEFAULT, SWT.DEFAULT, true, false));
		Iterator iter = displayableKeywordTypes.iterator();
		while (iter.hasNext()) {
			keywordsSearchType.add((String)iter.next());
		}
		
		keywordsSearchType.select(0);
		
		/* TODO: jpound - Allow multiple keywords to be selected (no combo) */
		keywordsSearchValue = new Combo(advancedSearchComposite, SWT.READ_ONLY);
		keywordsSearchValue.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		iter = options.keywordsIterator();
		while (iter.hasNext()) {
			keywordsSearchValue.add((String)iter.next());
		}
		
		/* Email and Numbering tab */
		TabItem emailAndNumbering = new TabItem(folder, SWT.DEFAULT);
		emailAndNumbering.setText("Email and Numbering");
		Composite emailAndNumberingComp = new Composite(folder, SWT.NONE);
		emailAndNumberingComp.setLayout(new GridLayout(2, true));
		emailAndNumberingComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		Composite email = new Composite(emailAndNumberingComp, SWT.NONE);
		email.setLayout(new GridLayout(1, true));
		email.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		Group emailComposite = new Group(email, SWT.SHADOW_NONE);
		emailComposite.setText("Email");
		emailComposite.setLayout(new GridLayout(1, false));
		emailComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		emailAndNumbering.setControl(emailAndNumberingComp);
		//emailComposite.setEnabled(false);
//		Label label = new Label(emailComposite, SWT.LEFT);
//		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, SWT.DEFAULT, false, false, 1, 1));
//		label.setText("Any of");
		Composite numberingAndVotesComp = new Composite(emailAndNumberingComp, SWT.NONE);
		numberingAndVotesComp.setLayout(new GridLayout(1, true));
		numberingAndVotesComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		Group votesComposite = new Group(numberingAndVotesComp, SWT.SHADOW_NONE);
		votesComposite.setText("Votes");
		votesComposite.setLayout(new GridLayout(2, false));
		votesComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		Group numberingComposite = new Group(numberingAndVotesComp, SWT.SHADOW_NONE);
		numberingComposite.setText("Numbering");
		numberingComposite.setLayout(new GridLayout(2, false));
		numberingComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		//numberingComposite.setEnabled(false);
		
		/* email */
		Composite checkBoxComposite = new Composite(emailComposite, SWT.NONE);
		checkBoxComposite.setLayout(new GridLayout(2, false));
		checkBoxComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Composite emailSearchComposite = new Composite(emailComposite, SWT.NONE);
		emailSearchComposite.setLayout(new GridLayout(1, false));
		emailSearchComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		bugOwner = createCheckBox(checkBoxComposite);
		bugOwner.setText("Bug Owner");
		bugOwner.setSelection(true);
		bugReporter = createCheckBox(checkBoxComposite);
		bugReporter.setText("Reporter");
		ccMember = createCheckBox(checkBoxComposite);
		ccMember.setText("CC Member");
		bugCommenter = createCheckBox(checkBoxComposite);
		bugCommenter.setText("Commenter");
		
		emailSearchType = new Combo(emailSearchComposite, SWT.READ_ONLY);
		emailSearchType.add("contains");
		emailSearchType.add("equals");
		emailSearchType.add("matches the regexp");
		emailSearchType.add("doesnt match the regexp");
		emailSearchType.select(0);	
		emailField = new Text(emailSearchComposite, SWT.SINGLE | SWT.BORDER);
		emailField.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		/* numbering */
		bugVotesSearchType = new Combo(votesComposite, SWT.READ_ONLY);
		bugVotesSearchType.add("Only bugs with vote count exceeding");
		bugVotesSearchType.select(0);
		bugVotesSearchType.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		bugVotes = new Text(votesComposite, SWT.SINGLE | SWT.BORDER);
		bugVotes.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		bugVotes.setTabs(5);
		
		bugNumbersSearchType = new Combo(numberingComposite, SWT.READ_ONLY);
		bugNumbersSearchType.add("Only include bugs numbered");
		bugNumbersSearchType.add("Exclude bugs numbered");
		bugNumbersSearchType.select(0);
		bugNumbersSearchType.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));
		bugNumbers = new Text(numberingComposite, SWT.SINGLE | SWT.BORDER);
		bugNumbers.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 2, 1));
		
				
		/* Bug Changes tab */
		TabItem bugChanges = new TabItem(folder, SWT.DEFAULT);
		bugChanges.setText("Bug Changes");
		Composite bugChangesComposite = new Composite(folder, SWT.NONE);
		GridLayout bugChangeLayout = new GridLayout(4, false);
		bugChangeLayout.marginHeight = 15;
		bugChangesComposite.setLayout(bugChangeLayout);
		bugChangesComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		bugChanges.setControl(bugChangesComposite);
		
		Label changeLable1 = new Label(bugChangesComposite, SWT.LEFT);
		changeLable1.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, SWT.DEFAULT, false, false));
		changeLable1.setText("Only include bugs changed between");
		
		bugChangeFrom = createTextField(bugChangesComposite);
		bugChangeFrom.setText("YYYY-MM-DD");
		bugChangeFrom.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent e) {
				if (bugChangeFrom.getText().equals("YYYY-MM-DD")) {
					bugChangeFrom.setText("");
				}
			}

			public void focusLost(FocusEvent e) {
				// this space intentionally left blank
			}
		});
		
		
		createLabel(bugChangesComposite, "and");
		bugChangeTo = createTextField(bugChangesComposite);
		bugChangeTo.setText("Now");
		
		Label changeLable2 = new Label(bugChangesComposite, SWT.LEFT);
		changeLable2.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, SWT.DEFAULT, false, false));
		changeLable2.setText("where one of the following was modified");
		bugChangeList = createList(bugChangesComposite);
		GridData smallVertData = new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1);
		smallVertData.heightHint = 50;
		bugChangeList.setLayoutData(smallVertData);
		bugChangeList.add("[Bug creation]");
		bugChangeList.add("Alias");
		bugChangeList.add("Assignee");
		bugChangeList.add("URL");
		bugChangeList.add("Severity");
		bugChangeList.add("Status");
		bugChangeList.add("CC list accessible?");
		bugChangeList.add("Component");
		bugChangeList.add("Ever confirmed?");
		bugChangeList.add("Keywords");
		bugChangeList.add("OS");
		bugChangeList.add("Priority");
		bugChangeList.add("Product");
		bugChangeList.add("QA Contact");
		bugChangeList.add("Hardware");
		bugChangeList.add("Reporter");
		bugChangeList.add("Reporter accessible?");
		bugChangeList.add("Resolution");
		bugChangeList.add("Summary");
		bugChangeList.add("Whiteboard");
		bugChangeList.add("Target Milestone");
		bugChangeList.add("Version");
		bugChangeList.add("Votes");
		
		Label changeLable3 = new Label(bugChangesComposite, SWT.LEFT);
		changeLable3.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, SWT.DEFAULT, false, false));
		changeLable3.setText("and the new value was");
		
		bugChangeNewValue = createTextField(bugChangesComposite);
		bugChangeNewValue.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false, 3, 1));
	}
	
	/**
	 * @param group
	 */
	private Combo createSearchTypeCombo(Composite group) {
		Combo combo = new Combo(group, SWT.READ_ONLY);
		Iterator iter = displayableSearchTypes.iterator();
		while (iter.hasNext()) {
			combo.add((String)iter.next());
		}
		
		combo.select(0);
		return combo;
	}

	/**
	 * 
	 * @param searchGroup
	 */
	private Text createTextField(Composite searchGroup) {
		Text textField = new Text(searchGroup, SWT.SINGLE | SWT.BORDER);
		textField.setLayoutData(SWTUtils.createHFillGridData());
		textField.addModifyListener(this);
		
		return textField;
	}

	/**
	 * 
	 * @param searchGroup
	 * @param text
	 */
	private void createLabel(Composite searchGroup, String text) {
		Label label = new Label(searchGroup, SWT.LEFT);
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, SWT.DEFAULT, false, false, 1, SWT.DEFAULT));
		label.setText(text);
	}
	
	/**
	 * @param group
	 */
	private List createList(Composite group) {
		GridData smallVertData = new GridData(SWT.FILL, SWT.FILL, true, true);
		smallVertData.heightHint = 100;
		smallVertData.widthHint = 100;
		
		List list = new List(group, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		list.setLayoutData(smallVertData);
		
		return list;
	}
	
	private Button createCheckBox(Composite c) {
		return new Button(c, SWT.CHECK);
	}
	
	/**
	 * @param items
	 */
	private void populateList(Iterator items, List list) {
		if (items == null || list == null) {
			return;
		}
		while (items.hasNext()) {
			list.add((String)items.next());
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		checkComplete();
	}
	
	private void checkComplete() {
		if (advancedView) {
			setComplete(true);
		}
		else if (summaryField.getText().equals("") && productList.getSelectionIndex() == -1) {
			setComplete(false);
			return;
		}
		setComplete(true);
	}
	
	/* A class to populate one List (dependant) based on the selection of another (source) */
	private class BugzillaListSelectionListener implements SelectionListener {
		private List source, dependant;
		private int optionIndex;
		
		public BugzillaListSelectionListener(List source, List dependant, int optionIndex) {
			this.source = source;
			this.dependant = dependant;
			this.optionIndex = optionIndex;
		}
		
		public void widgetSelected(SelectionEvent e) {
			
			int[] sourceSelection = source.getSelectionIndices();
			java.util.List option = new LinkedList();
						
			if (sourceSelection.length > 0) {
				
				for (int i = 0; i < sourceSelection.length; i++) {
					java.util.List[] allOptions = options.getProductOptions(source.getItem(sourceSelection[i])); 
					if (allOptions == null) {
						continue;
					}
					
					option.addAll(allOptions[optionIndex]);
				}
				if (option.isEmpty()) {
					dependant.removeAll();
					return;
				}
			}
			
			/* Case insensitive compare */
			Collections.sort(option, new Comparator() {
				public int compare(Object o1, Object o2) {
					return (((String)o1).toUpperCase()).compareTo(((String)o2).toUpperCase());
				}
			});
			
			Iterator optionIter = option.iterator();
			dependant.removeAll();
			while (optionIter.hasNext()) {
				dependant.add((String)optionIter.next());
			}
			
			setComplete(true);
		}
		public void widgetDefaultSelected(SelectionEvent e) {
			widgetSelected(e);
		}
	}
}