/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.model;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.team.bugs.model.EventHandler;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugs.model.IGenericModel;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.Util;

/**
 * 
 */
public abstract class BugzillaModel implements IGenericModel, IAdaptable {
    
    private BugzillaModel fParent;
    private boolean fRefreshing;
    private String fPath;

    public BugzillaModel() {
        super();
    }
    
    public IBugProvider getProvider() {
        if (fParent == null) return null;
        return fParent.getProvider();
    }
    
    public BugzillaProvider getBugzillaProvider() {
        return (BugzillaProvider)getProvider();
    }

    public BugzillaModel getParent() {
        return fParent;
    }

    /**
     * Set the parent element of this model element, and reset its path. This 
     * method should not be called in order to add models to a BugzillaContainer;
     * use <code>BugzillaContainer.addChild()</code> instead.
     */
    protected void setParent(BugzillaModel parent) {
        
        if (fParent == parent)
            return;
        
        fParent= parent;
        resetPath();
    }

    /**
     * Reset the path of this model element. A new path is generated and cached
     * on the next call to <code>getPath()</code>. Subclasses with children
     * may extend this method in order to correctly reset the path of children
     * elements.
     */
    protected void resetPath() {
        fPath= null;
    }

    public boolean isRefreshing() {
        return fRefreshing;
    }
    
    public boolean isReadOnly() {
    	return false;
    }
    
    public void setRefreshing(boolean refreshing) {
       fRefreshing= refreshing;
       fireModelChanged();
    }

    public Object getAdapter(Class adapter) {
        return Platform.getAdapterManager().getAdapter(this, adapter);
    }
    
    /**
     * Update the model element displayed in any views. This method
     * is invoked at the end of a refresh cycle and must be invoked
     * in the UI thread. By default, any refreshing visual feedback is
     * removed and a model event change is fired. This method must be called
     * for all elements that have been requested to refresh, even if the refresh
     * failed. If this is not done, the refresh visual feedback will not be cleared.
     */
    public void update() {
        setRefreshing(false); // fire model changed is called here.
    }
    
    public void accept(IBugzillaModelVisitor visitor) throws BugzillaException {
        visitor.visit(this);
    }

    private EventHandler getEventHandler() {
        return getProvider() != null ? getProvider().getEventHandler() : null;
    }
    
    /**
     * Notify any listeners that this model element as changed
     */
    protected void fireModelChanged() {
        EventHandler eventHandler = getEventHandler();
		if (eventHandler != null)
		    eventHandler.elementsChanged(new IGenericModel [] { this });
    }
    
    /**
     * Notify any listeners that the given child
     * was removed from this element.
     * @param child the removed child
     */
    protected void fireChildRemoved(BugzillaModel child) {
        EventHandler eventHandler = getEventHandler();
        if (eventHandler != null)
            eventHandler.elementsRemoved(this, new IGenericModel [] { child });
    }

    protected void fireChildrenRemoved(BugzillaModel [] children) {
        EventHandler eventHandler = getEventHandler();
        if (eventHandler != null)
            eventHandler.elementsRemoved(this, children);
    }

    /**
     * Notify any listsners that the child was added to this container
     * @param child the added child
     */
    protected void fireChildAdded(BugzillaModel child) {
        EventHandler eventHandler = getEventHandler();
        if (eventHandler != null) 
            eventHandler.elementsAdded(this, new IGenericModel [] { child });
    }
    
    protected void fireChildrenAdded(BugzillaModel [] children) {
        EventHandler eventHandler = getEventHandler();
        if (eventHandler != null) 
            eventHandler.elementsAdded(this, children);
    }

    
    /**
     * Return the id of the model element. An id uniquely
     * identifies a model element. The id of a model element
     * must never change (although once removed from the model,
     * it can be changed and readded). The id is used to build
     * a path that is used for equality checking and hashcode
     * generation. This is done to allow equals model elements to
     * appear multiple places in the model.
     * <p>
     * The id of an element must not contain any forward slashes
     * (/) as this is the path separator.
     * @return the path id of the model element
     */
    public abstract String getID();
    
    /**
     * Return the full path of this model element in the model. If the cached path
     * is null, the path is regenerated by appending the id of this element to the 
     * path of its parent.
     * @return the full path of this model element in the model
     */
    public String getPath() {
        if (fPath == null) {
            final Object parent = getParent();
            if (parent instanceof BugzillaModel) {
                fPath= Util.appendPath(((BugzillaModel)parent).getPath(), getID());
            } else { 
                fPath= getID();
            }
        }
        return fPath;
    }
    
    public boolean hadErrors() {
        return false;
    }
}
