/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.model;

import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.team.bugs.extension.BugsException;
import org.eclipse.team.bugs.model.EventHandler;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaImages;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.Util;
import org.eclipse.team.bugzilla.datatypes.AttachmentDescription;
import org.eclipse.team.bugzilla.datatypes.BugzillaConfiguration;
import org.eclipse.team.bugzilla.datatypes.DataCache;
import org.eclipse.team.bugzilla.datatypes.IBugRefreshListener;
import org.eclipse.team.bugzilla.datatypes.QueryURL;
import org.eclipse.team.bugzilla.operations.BugzillaOperations;
import org.eclipse.team.bugzilla.operations.CachedOperations;
import org.eclipse.team.bugzilla.operations.RefreshOperation;
import org.eclipse.team.bugzilla.persistence.PersistenceManager;
import org.eclipse.ui.progress.UIJob;

/**
 * 
 */
public class BugzillaProvider extends BugzillaContainer implements IBugProvider, LocationListener, IBugRefreshListener {

    private final CachedOperations fOperations;
    private BugzillaHistoryFolder fHistoryFolder;
    private EventHandler fEventHandler;
    private DataCache fCache;
    private ImageDescriptor icon;
    
    public BugzillaProvider(EventHandler eventHandler, BugzillaConfiguration configuration) {
        super();
        fEventHandler= eventHandler;
        fOperations= new CachedOperations(configuration);
        fCache = new DataCache(this);
        fCache.addRefreshListener(this);
        
        fCache.refreshQueryOptions();
		icon = null;
    }
    
    public BugzillaConfiguration getConfiguration() {
        return fOperations.getConfiguration();
    }
    
    public CachedOperations getOperations() {
        return fOperations;
    }
    
    public EventHandler getEventHandler() {
        return fEventHandler;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IBugProvider#setEventHandler(org.eclipse.team.bugs.model.EventHandler)
     */
    public void setEventHandler(EventHandler eventHandler) {
        fEventHandler= eventHandler;
    }

    /**
     * This provider has been added to a view. Persist the model
     */
    public void added(Object parent) {
        PersistenceManager.providerAdded(this);
    }

    public void removed(Object parent) {
        PersistenceManager.providerRemoved(this);
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#getParent()
     */
    public BugzillaModel getParent() {
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#getProvider()
     */
    public IBugProvider getProvider() {
        return this;
    }
    
	public void changing(LocationEvent event) {	
	}

	public void changed(LocationEvent event) {
		BugzillaHistoryFolder history  = getHistoryFolder();
		if(history == null)
		    return;

		try {
		    final String reportID = getOperations().getReportIdFromURL(event.location);
		    if (!(reportID == null || history.contains(reportID))) {
		        final BugzillaReport report=  new BugzillaReport(history, reportID);
		        new RefreshOperation(null, new IRemoteModel[] { report }).run();
		    } else {
		        final String queryID = getOperations().getQueryURL(event.location);
		        if (!(queryID == null || history.contains(reportID))) {
		            final BugzillaQuery query= new BugzillaQuery(history, queryID);
		            new RefreshOperation(null, new IRemoteModel[] { query }).run();
		        }
		    }
		} catch (BugsException e) {				
		    e.printStackTrace();
		} catch (InvocationTargetException e) {
		    BugzillaPlugin.log("Error while refreshing", e.getTargetException());
		} catch (InterruptedException e) {
		    // Ignore
		}		
	}
	
	private BugzillaHistoryFolder getHistoryFolder() {
		if (fHistoryFolder == null) {
			BugzillaModel[] children = getChildren();
			for (int i = 0; i < children.length; i++) {
                BugzillaModel element = children[i];
				if (element instanceof BugzillaHistoryFolder) {
					fHistoryFolder = (BugzillaHistoryFolder) element;
				}
			}
		}
		return fHistoryFolder;
	}

    public DataCache getDataCache() {
        return fCache;
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.datatypes.IBugRefreshListener#reportDataChange(org.eclipse.team.bugzilla.model.BugzillaProvider, java.lang.String[])
     */
    public void reportDataChange(final BugzillaProvider provider, final String[] ids) {
        UIJob job = new UIJob("Udating view") {
            public IStatus runInUIThread(IProgressMonitor monitor) {
                try {
                    updateReports(provider, ids);
                } catch (BugzillaException e) {
                    return e.getStatus();
                }
                return Status.OK_STATUS;
            }
        };
        job.setSystem(true);
        job.schedule();
    }

    protected void updateReports(BugzillaContainer container, final String[] ids) throws BugzillaException {
        container.accept(new BugzillaReportVisitor() {
            protected void visitReport(BugzillaReport report) throws BugzillaException {
                for (int j = 0; j < ids.length; j++) {
                    String id = ids[j];
	                if (report.getID().equals(id)) {
	                    report.update();
	                }
                }

            }
        });
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.datatypes.IBugRefreshListener#attachmentsChanged(org.eclipse.team.bugzilla.model.BugzillaProvider, org.eclipse.team.bugzilla.datatypes.AttachmentDescription[])
     */
    public void attachmentsChanged(final BugzillaProvider provider, final AttachmentDescription[] descs) {
        UIJob job = new UIJob("Udating view") {
            public IStatus runInUIThread(IProgressMonitor monitor) {
                try {
                    updateAttachments(provider, descs);
                } catch (BugzillaException e) {
                    return e.getStatus();
                }
                return Status.OK_STATUS;
            }
        };
        job.setSystem(true);
        job.schedule();
    }

    /**
     * @param provider
     * @param descs
     * @throws BugzillaException
     */
    protected void updateAttachments(BugzillaProvider provider, final AttachmentDescription[] descs) throws BugzillaException {
        provider.accept(new BugzillaReportVisitor() {
            protected void visitReport(BugzillaReport report) throws BugzillaException {
                for (int j = 0; j < descs.length; j++) {
                    AttachmentDescription desc = descs[j];
	                if (report.getID().equals(desc.getString(AttachmentDescription.BUG_ID))) {
	                    report.update();
	                }
                }

            }
        });
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.datatypes.IBugRefreshListener#queryChange(org.eclipse.team.bugzilla.model.BugzillaProvider, org.eclipse.team.bugzilla.datatypes.QueryURL[])
     */
    public void queryChange(final BugzillaProvider provider, final QueryURL[] datas) {
        UIJob job = new UIJob("Udating view") {
            public IStatus runInUIThread(IProgressMonitor monitor) {
                try {
                    updateQueries(provider, datas);
                } catch (BugzillaException e) {
                    return e.getStatus();
                }
                return Status.OK_STATUS;
            }
        };
        job.setSystem(true);
        job.schedule();
    }

    /**
     * @param provider
     * @param datas
     * @throws BugzillaException
     */
    protected void updateQueries(BugzillaProvider provider, final QueryURL[] datas) throws BugzillaException {
        provider.accept(new QueryVisitor() {
            protected void visitQuery(BugzillaQuery query) throws BugzillaException {
                for (int i = 0; i < datas.length; i++) {
                    QueryURL url = datas[i];
                    if (query.getQueryUrl().equals(url)) {
                        query.update();
                    }
                }
            }
        });
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaContainer#getName()
     */
    public String getName() {
        return fOperations.getConfiguration().getName();
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#getId()
     */
    public String getID() {
        return Util.asValidPathSegment(fOperations.getConfiguration().getBaseURL());
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#getPath()
     */
    public String getPath() {
        return Util.asRelativePath(getID());
    }
    
    public String getBaseURL() {
    	return fOperations.getConfiguration().getBaseURL();
    }

    public void setLoginCookie(String cookie) {
        getConfiguration().setLoginCookie(cookie);      
    }
    public String getLoginCookie() {
        return getConfiguration().getLoginCookie();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaContainer#canModifyChildren()
     */
    public boolean canModifyChildren() {
        return true;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#isReadOnly()
     */
    public boolean isReadOnly() {
        return true;
    }

	public ImageDescriptor getImageDescriptor() {
		if (icon == null) {
			if ((icon = BugzillaOperations.getBugzillaIcon(this.getBaseURL())) == null) {
				icon = BugzillaImages.BUGZILLA;
			}
		}
		
		return icon;
	}

	public void setImageDescriptor(ImageDescriptor i) {
		icon = i;
	}
	
	public void setImageDescriptor(String url) throws MalformedURLException {
		icon = ImageDescriptor.createFromURL(new URL(url));
	}
}