/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.model;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.Util;
import org.eclipse.team.bugzilla.datatypes.QueryData;
import org.eclipse.team.bugzilla.datatypes.QueryURL;
import org.eclipse.team.bugzilla.datatypes.ReportData;
import org.eclipse.team.bugzilla.operations.RefreshOperation;

/**
 * A query that can be executed against a bugzilla bug database.
 */
public class BugzillaQuery extends BugzillaContainer implements IRemoteModel {
    
    private QueryURL queryData;
    private String name = null;

    /**
     * Validate whether the given text is a valid query URL
     * @param text the text
     * @return whether the given text is a valid query URL
     */
    public static IStatus validateQueryString(String text) {
        // TODO Query validation
        return Status.OK_STATUS;
    }
    
    /**
     * Create a query
     */
    public BugzillaQuery(BugzillaContainer parent, String queryString) {
        super();
        this.queryData = new QueryURL(queryString);
        if (parent != null)
            parent.addChildren(new BugzillaModel [] { this });
        if (queryData.isNamedQuery()) {
            name = queryData.getName();
        }
    }
    
    /**
     * Create a query from an existing QueryURL
     */
    public BugzillaQuery(BugzillaContainer parent, QueryURL query) {
        super();
        this.queryData = query;
        if (parent != null)
            parent.addChildren(new BugzillaModel [] { this });
        if (queryData.isNamedQuery()) {
            name = queryData.getName();
        }
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IRemoteModel#hasLocalContent()
     */
    public boolean hasLocalContent() {
        return getChildren().length > 0;
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IRemoteModel#fetchRemoteState(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void refresh(IProgressMonitor monitor) throws BugzillaException {
    	getBugzillaProvider().getDataCache().refreshQuery(new QueryURL[] { queryData }, monitor);	
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugs.model.IRemoteModel#update(java.lang.Object)
     */
    public void update() {
        QueryData data = getBugzillaProvider().getDataCache().getQueryData(getQueryUrl());
        if (data == null) {
            return;
        }
        BugzillaModel[] children = getChildren();
        removeChildren(children);

        ReportData[] datas = data.getReportDatas();
        final BugzillaReport [] reports= new BugzillaReport[datas.length]; 
        for (int i = 0; i < datas.length; i++) {
            ReportData reportData = datas[i];
            reports[i]= new BugzillaReport(null, reportData.getString(ReportData.ID));
        }
        addChildren(reports);
        super.update();
    }

    public String getName() {
        if (this.name == null) {
            this.name = queryData.toUrlString();
        }
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
        fireModelChanged();
    }
    
    public QueryURL getQueryUrl() {
        return queryData;
    }
    
    public void setQueryUrl(QueryURL q) {
    	if (q == null || q.equals(queryData)) {
    		return;
    	}

    	// set the name to the new url if the query hasn't been renamed
    	if (getName().equals(queryData.toUrlString())) {
    		setName(q.toUrlString());
    	}
    	queryData = q;
    	
    	try {
			new RefreshOperation(null, new IRemoteModel[] { this }).run();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#getId()
     */
    public String getID() {
        return "Q" + Util.asValidPathSegment(queryData.toUrlString()); //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#isReadOnly()
     */
    public boolean isReadOnly() {
        return false;
    }

    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaContainer#canModifyChildren()
     */
    public boolean canModifyChildren() {
        return false;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaContainer#setRefreshing(boolean)
     */
    protected void setChildrenRefreshing(boolean refreshing) {
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.IRemoteModel#needsRefresh()
     */
    public boolean needsRefresh() {
        return getBugzillaProvider().getDataCache().getQueryData(getQueryUrl()) == null;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "query " + getName();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.team.bugzilla.model.BugzillaModel#hadErrors()
     */
    public boolean hadErrors() {
        if (getBugzillaProvider() == null)
            return false;
        return getBugzillaProvider().getDataCache().hadErrorsForQuery(getQueryUrl());
    }
}
