/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.operations;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaFactory;
import org.eclipse.team.bugzilla.datatypes.AttachmentData;
import org.eclipse.team.bugzilla.datatypes.AttachmentDescription;
import org.eclipse.team.bugzilla.datatypes.BugzillaConfiguration;
import org.eclipse.team.bugzilla.datatypes.QueryData;
import org.eclipse.team.bugzilla.datatypes.QueryURL;
import org.eclipse.team.bugzilla.datatypes.ReportData;
import org.eclipse.team.bugzilla.xml.Parser;
import org.eclipse.team.bugzilla.xml.VersionHandler;


public abstract class BugzillaOperations {

	protected static String bugScript; /* TODO: jpound - part of redirect workaround for gcj */
	//TODO:private static final String BAD_XML_HEADER = "<?xml version=\"1.0\" ?>";
    public static BugzillaOperations getOperations(IProgressMonitor monitor, BugzillaConfiguration configuration) throws BugzillaException {
		int version = configuration.getVersion();
    	if (version < 0) {
    		version = getVersion(monitor, configuration);
    		configuration.setVersion(version);
    	}
		return create(version);
	}
       
    public static BugzillaOperations create(int version) {
        switch (version) {
        case BugzillaFactory.VERSION_2_16:
        	return new BugzillaOperations_2_16();
        case BugzillaFactory.VERSION_2_18:
        	return new BugzillaOperations_2_18();
            
        }
        return null;
    }
	
    public static int getVersion(IProgressMonitor monitor, BugzillaConfiguration configuration) throws BugzillaException {
        // find the version by requesting an invalid bug.
		String url = configuration.getBaseURL();
		if (!url.endsWith("/")) {
			url += "/";
		}
		try {
			final HTTPConnection connection= new HTTPConnection(url + "xml.cgi?id=1");
			final String xml= connection.getString(monitor);
			final Parser parser= new Parser(false, false);
			final VersionHandler handler= new VersionHandler();
			parser.parse(xml, handler);
			bugScript = "xml.cgi?id=";
			return handler.getVersion();
		} catch (Exception e) {
			/* FIXME: jpound - work around for gcj not handling the xml.cgi redirect in bugilla 2.18 */
			bugScript = "show_bug.cgi?ctype=xml&id=";
			final HTTPConnection connection= new HTTPConnection(url + bugScript + "1");
			final String xml= connection.getString(monitor);
			final Parser parser= new Parser(false, false);
			final VersionHandler handler= new VersionHandler();
			parser.parse(xml, handler);
			return handler.getVersion();
		}
    }
	
	public static ImageDescriptor getBugzillaIcon(String url) {
		try {
			BufferedReader read = new BufferedReader(new InputStreamReader(new URL(url).openStream()));
			String line;
			String image;
			url = url.endsWith("/") ? url : url + "/";	
			Pattern iconFlag = Pattern.compile(".*rel=\".*icon\".*");
			
			while ((line = read.readLine()) != null) {
				
				Matcher m = iconFlag.matcher(line);
				if (m.matches()) {
					image = parseUrl(read, line, url);
					read.close();
					return (image == null) ? null : ImageDescriptor.createFromURL(new URL(image));
				}
			}
			read.close();
			/* Attempt to find the default icon */
			URL base = new URL(url);
			URL iconUrl = new URL(base.getProtocol() + "://" + base.getHost() + "/" + "favicon.ico");

			ImageDescriptor defaultIcon = getBugzillaIconImageDescriptor(iconUrl);
			
			if ((defaultIcon != null) && (defaultIcon.getImageData() != null)) {
				return defaultIcon;
			}
			else {
				// Because a lot of bug systems are on a separate sub-domain,
				// try the main hostname for the favicon before defaulting.
				String oldHost = base.getHost();
				String[] oldHostElements = oldHost.split("\\.");
				String newHost = "";
				for (int i = 1; i < oldHostElements.length; i++) {
					if (i == 1) {
						newHost += oldHostElements[i];
					} else
						newHost += "." + oldHostElements[i];
				}
				
				// Try the same protocol but stripping the first element of the hostname
				iconUrl = new URL(base.getProtocol() + "://" + newHost + "/" + "favicon.ico");
				
				defaultIcon = getBugzillaIconImageDescriptor(iconUrl);
				if ((defaultIcon != null) && (defaultIcon.getImageData() != null)) {
					return defaultIcon;
				}
				
				// Try the first element of the hostname but non-SSL
				iconUrl = new URL("http://" + newHost + "/" + "favicon.ico");
				
				defaultIcon = getBugzillaIconImageDescriptor(iconUrl);
				if ((defaultIcon != null) && (defaultIcon.getImageData() != null)) {
					return defaultIcon;
				}
			}
			
		} catch (MalformedURLException e) {
			System.err.println("BugzillaOperations: Malformed URL when fetching icon");
		} catch (IOException e) {
			//System.err.println("BugzillaOperations: IOException exception when fetching icon");
		}
		return null;
	}
	
	private static ImageDescriptor getBugzillaIconImageDescriptor(URL iconUrl) throws IOException {
		URLConnection con = iconUrl.openConnection();
		int len = con.getContentLength();
		
		if (len <= 0) {
			return null;
		}
		
		ImageDescriptor desc = ImageDescriptor.createFromURL(iconUrl);
		
		/* Servers that have custom file-not-found pages sometimes trick
		 * us into believing that page is an image, here we check to make
		 * sure the image is valid.
		 */
		if (desc != null){
			try {
				desc.getImageData();
			}catch (SWTException e){
				if (e.code == SWT.ERROR_INVALID_IMAGE || e.code == SWT.ERROR_UNSUPPORTED_FORMAT)
					desc = null;
			}			
		}
		
		return desc;
	}
	
	private static String parseUrl(BufferedReader read, String line, String url) throws IOException {
		String hrefFlag = "href=\"";
		int index;
		do {
			if ((index = line.indexOf(hrefFlag)) >= 0) {
				/* keep breaking off begining part of string to get icon url */
				line = line.substring(index + hrefFlag.length());
				url += line.substring(0, line.indexOf("\""));
				
				return url;
			}
		} while ((line = read.readLine()) != null);
		
		return null;
	}
        
    public abstract ReportData getReportData(IProgressMonitor monitor, BugzillaConfiguration configuration, String id) throws BugzillaException;
    
    public abstract QueryData getQueryData(IProgressMonitor monitor, BugzillaConfiguration configuration, QueryURL queryURL) throws BugzillaException;

	public abstract AttachmentData getAttachment(IProgressMonitor monitor, BugzillaConfiguration configuration, AttachmentDescription desc) throws BugzillaException;
	
	public abstract String getReportURL(BugzillaConfiguration configuration, String reportID);

	public abstract String getCommentURL(BugzillaConfiguration configuration, String reportID, int commentNbr);
	
	public abstract String getAttachmentURL(BugzillaConfiguration configuration, String attachmentID);
	
	public abstract String getReportIDFromURL(BugzillaConfiguration configuration, String url);
	
	public abstract String getQueryURL(BugzillaConfiguration configuration, String url);
}