/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.bugzilla.operations;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.InterruptedIOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.internal.core.streams.PollingInputStream;
import org.eclipse.team.internal.core.streams.TimeoutInputStream;


public class HTTPConnection {
    
    private static class ConnectingThread extends Thread {

        private final String fCookie;
        private final URL fURL;
        private final int fTimeOut;
        
        private InputStream fStream;
        private String fContentType;
        private IOException fException;

        private ConnectingThread(String loginCookie, URL url, int timeOut) {
            fCookie = loginCookie;
            fURL = url;
            fTimeOut= timeOut;
        }

        public void run() {
            try {
                final URLConnection connection = fURL.openConnection();
                if (fCookie != null)
                    connection.setRequestProperty("Cookie", fCookie); //$NON-NLS-1$
                connection.connect();
                final InputStream stream = connection.getInputStream();
                synchronized (this) {
                    if (interrupted()) {
                        // we we're either cancelled or timed out so just destroy the process
                        stream.close();
                    } else {
                        fStream= stream;
                        fContentType= connection.getContentType();
                    }
                }
            } catch (IOException e) {
                fException = e;
            }
        }
        
        public synchronized void abort() {
            if (isAlive()) {
                interrupt();
            }
        }
        
        public synchronized void cleanUp() {
            try {
                if (fStream != null) {
                    fStream.close();
                }
            } catch (IOException e) {
                // ignore
            }
        }
        
        public InputStream getStream(IProgressMonitor monitor) throws IOException {
            start();
            
            // Wait the appropriate number of seconds
            for (int i = 0; i < fTimeOut; i++) {
                try {
                    // wait for the thread to complete or 1 second, which ever comes first
                    join(1000);
                } catch (InterruptedException e) {
                    // I think this means the thread was interupted but not necessarily timed out
                    // so we don't need to do anything
                }
                // if the user cancelled, clean up before preempting the operation
                if (monitor.isCanceled()) {
                    abort();
                    cleanUp();
                    throw new OperationCanceledException();
                }
            }
            // If the thread is still running (i.e. we timed out) signal that it is too late
            abort();
            
            if (fException != null) {
                throw fException;
            }
            if (fStream == null) {
                throw new InterruptedIOException("Time out waiting for " + fURL.toExternalForm()); //$NON-NLS-1$
            }
            return fStream;
        }
        
        public synchronized IOException getException() {
            return fException;
        }
        
        public synchronized String getContentType() {
            return fContentType;
        }
    }

    private final String fURL;
    private final String fCookie;

    private String fContentType;
    
    public HTTPConnection(String url) {
        this(url, null);
    }
    
    public HTTPConnection(String url, String cookie) {
        fURL= url;
        fCookie= cookie;
    }
    
    /*
    public InputStream getInputStream(IProgressMonitor monitor) throws BugzillaException {
    	InputStream in = null;
    	try {
            final ConnectingThread thread = new ConnectingThread(fCookie, new URL(fURL), 60);
            in = thread.getStream(monitor);
            fContentType= thread.getContentType();
    	} catch (Exception e) {
            handleError(e);
        }
    	return in;
    }
    */
    public String getString(IProgressMonitor monitor) throws BugzillaException {
        final StringWriter stringWriter = new StringWriter();
        try {
            final ConnectingThread thread = new ConnectingThread(fCookie, new URL(fURL), 60);
            final InputStream stream = thread.getStream(monitor);
            fContentType= thread.getContentType();
            final BufferedReader in = new BufferedReader(new InputStreamReader(getTimedInputStream(stream, monitor)));
            final PrintWriter writer = new PrintWriter(stringWriter);
            try {
                String line;
                while ((line = in.readLine()) != null && !monitor.isCanceled()) {
                    writer.println(line);
                }
            } finally {
                in.close();
            }
        } catch (Exception e) {
            handleError(e);
        }
        return stringWriter.toString();
    }
    
    public byte [] getBytes(IProgressMonitor monitor) throws BugzillaException {
        final ByteArrayOutputStream out= new ByteArrayOutputStream();
        try {
            final ConnectingThread thread = new ConnectingThread(fCookie, new URL(fURL), 60);
            final InputStream in = thread.getStream(monitor);
            fContentType= thread.getContentType();
            try {
                byte[] buffer = new byte[1024];
                int length;
                while ((length = in.read(buffer)) >= 0 && !monitor.isCanceled())
                    out.write(buffer, 0, length);
            } finally {
                out.flush();
                in.close();
            }
        } catch (Exception e) {
            handleError(e);
        }
        return out.toByteArray();
    }
    
    public String getContentType() {
        return fContentType;
    }
    
    public InputStream getTimedInputStream(InputStream stream, IProgressMonitor monitor) {
        return new PollingInputStream(new TimeoutInputStream(stream,
                8192 /*bufferSize*/, 2000 /*readTimeout*/, -1 /*closeTimeout*/), 60, monitor);
    }
    
    private void handleError(Exception e) throws BugzillaException {
        
        if (e instanceof MalformedURLException)
            throw new BugzillaException(BugzillaException.IO_ERROR, e.getMessage(), e);
        
        if (e instanceof UnknownHostException) 
            throw new BugzillaException(BugzillaException.IO_ERROR, "Unknown host: " + e.getMessage(), e);
        
        if (e instanceof FileNotFoundException)
            throw new BugzillaException(BugzillaException.IO_ERROR, "File not found: " + e.getMessage(), e);
            
        if (e instanceof IOException) 
            throw new BugzillaException(BugzillaException.IO_ERROR, "IO Error: " + e.getMessage(), e);
            
        throw new BugzillaException("Unknown Error: " + e, e);
    }
}

