/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.persistence;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.team.bugs.extension.BugsException;
import org.eclipse.team.bugs.model.IBugProvider;
import org.eclipse.team.bugs.model.IGenericModel;
import org.eclipse.team.bugs.model.IProviderListener;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.adapters.BugzillaAdapterFactory;
import org.eclipse.team.bugzilla.model.BugzillaContainer;
import org.eclipse.team.bugzilla.model.BugzillaFolder;
import org.eclipse.team.bugzilla.model.BugzillaHistoryFolder;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.BugzillaProvider;
import org.eclipse.team.bugzilla.model.BugzillaQuery;
import org.eclipse.team.bugzilla.model.BugzillaReport;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

/**
 * 
 */
public class PersistenceManager {
    
    /*
     * The preference node in the instance scope where the providers are persisted
     */
    private static final String PROVIDER_STORE_ID = "providers";  //$NON-NLS-1$
    
    protected static final BugzillaModel [] EMPTY_ARRAY= new BugzillaModel[0];
    
    /**
     * The key for the model type preference.
     */
    public final static String MODEL_TYPE= "org.eclipse.team.bugzilla.modelType"; //$NON-NLS-1$
    
    public final static String ELEMENT_ID = "org.eclipse.team.bugzilla.elementId"; //$NON-NLS-1$

    /**
     * IDs for the different persistent model types.
     */
    public final static String PROVIDER= "provider"; //$NON-NLS-1$
    public final static String FOLDER= "folder"; //$NON-NLS-1$
    public final static String HISTORY= "history"; //$NON-NLS-1$
    public final static String REPORT= "report"; //$NON-NLS-1$
    public final static String QUERY= "query"; //$NON-NLS-1$

    private final static Map type2classMap;

    /*
     * Listener that is registered with each provider to update 
     * the store when the model changes
     */
    private static IProviderListener listener = new IProviderListener() {
        /* (non-Javadoc)
         * @see org.eclipse.team.bugs.model.IProviderListener#modelAdded(org.eclipse.team.bugs.model.IGenericModel, org.eclipse.team.bugs.model.IGenericModel)
         */
        public void elementsAdded(IGenericModel parent, IGenericModel[] children) {
            elementsChanged(children);
        }

        /* (non-Javadoc)
         * @see org.eclipse.team.bugs.model.IProviderListener#modelChanged(org.eclipse.team.bugs.model.IGenericModel)
         */
        public void elementsChanged(IGenericModel[] elements) {
            for (int i = 0; i < elements.length; i++) {
                final BugzillaModel model = (BugzillaModel)elements[i];
                final IPersistenceAdapter adapter = getPersistenceAdapter(elements[i]);
                if (adapter != null)
                    adapter.storeElement(model, false);
            }
            final BugzillaProvider provider= ((BugzillaModel)elements[0]).getBugzillaProvider();
            try {
                AbstractPersistenceAdapter.getPreferences(provider).flush();
            } catch (BackingStoreException e) {
                BugzillaPlugin.log(BugzillaException.asBugsException(e));
            }
        }

        /* (non-Javadoc)
         * @see org.eclipse.team.bugs.model.IProviderListener#modelRemoved(org.eclipse.team.bugs.model.IGenericModel, org.eclipse.team.bugs.model.IGenericModel)
         */
        public void elementsRemoved(IGenericModel parent, IGenericModel[] children) {
            for (int i = 0; i < children.length; i++) {
                final BugzillaModel model = (BugzillaModel)children[i];
                final IPersistenceAdapter adapter = getPersistenceAdapter(parent);
                if (adapter != null)
                    adapter.purgeChild((BugzillaModel)parent, model, false);
            }
            try {
                AbstractPersistenceAdapter.getPreferences(parent).flush();
            } catch (BackingStoreException e) {
                BugzillaPlugin.log(BugzillaException.asBugsException(e));
            }
        }
    };

    
    static {
        type2classMap = new HashMap();
        type2classMap.put(PROVIDER, BugzillaProvider.class);
        type2classMap.put(FOLDER, BugzillaFolder.class);
        type2classMap.put(HISTORY, BugzillaHistoryFolder.class);
        type2classMap.put(REPORT, BugzillaReport.class);
        type2classMap.put(QUERY, BugzillaQuery.class);
    }
    
    /**
     * Return the root preferences node whose children are providers
     * @return the root preferences node whose children are providers
     */
    /* package */ static Preferences getRootNode() {
        return new InstanceScope().getNode(BugzillaPlugin.ID).node(PROVIDER_STORE_ID);
    }
    
    /**
     * Restore the providers from the preference store and return the restored providers
     * @return the restored providers
     */
    public static IBugProvider[] restoreProviders() {
        BugzillaModel[] providers = restoreChildren(null /* no parent */, getRootNode());
        IBugProvider[] result = new IBugProvider[providers.length];
        System.arraycopy(providers, 0, result, 0, providers.length);
        return result;
    }
    
    /**
     * Return the persistence adapter for the given element 
     * or <code>null</code> if there isn't one.
     * @param element the element
     * @return the persistence adapter for the given element 
     * or <code>null</code>
     */
    public static IPersistenceAdapter getPersistenceAdapter(Object element) {
        if (element instanceof IAdaptable) {
	        Object adapter = ((IAdaptable)element).getAdapter(IPersistenceAdapter.class);
	        if (adapter instanceof IPersistenceAdapter)
	            return (IPersistenceAdapter)adapter;
        }
        return null;
    }
    
    /*
     * Method used by persistence adapters to restore a bugzilla model
     */
    /* package */ static BugzillaModel restoreChild(BugzillaContainer parent, Preferences node) {
        final String type= node.get(MODEL_TYPE, ""); //$NON-NLS-1$
        final String id = node.get(ELEMENT_ID, ""); //$NON-NLS-1$
        IPersistenceAdapter adapter = getPersistenceAdapter(type);
        if (adapter != null) {
            try {
                BugzillaModel element = adapter.restoreElement(parent, node, id);
	            return element;
            } catch (BugzillaException e) {
                BugzillaPlugin.log(e);
            }
        }
        return null;
    }
    
    /**
     * Restore the childen of the given element.
     * @param parent the element
     * @param node the node for the element that may contain child nodes
     * representing children of the element
     * @return the restored children
     */
    /* package */ static final BugzillaModel[] restoreChildren(BugzillaContainer parent, Preferences node) {
        final String [] childrenNames;
        try {
            childrenNames= node.childrenNames();
        } catch (BackingStoreException e) {
            BugzillaPlugin.log(BugsException.asBugsException(e));
            return EMPTY_ARRAY;
        }
        final ArrayList children= new ArrayList();
        for (int i = 0; i < childrenNames.length; i++) {
            final String childName= childrenNames[i];
            final Preferences childNode= node.node(childName);
            final BugzillaModel child= restoreChild(parent, childNode);
            if (child != null)
                children.add(child);
        }
        return (BugzillaModel[]) children.toArray(new BugzillaModel[children.size()]);
    }
    
    private static IPersistenceAdapter getPersistenceAdapter(String type) {
        Class typeClass = (Class)type2classMap.get(type);
        if (typeClass != null) {
            Object adapter = BugzillaAdapterFactory.getPersistenceAdapter(typeClass);
            if (adapter instanceof IPersistenceAdapter)
                return (IPersistenceAdapter)adapter;
        }
        return null;
    }

    /**
     * The provider has been added so persist it's state
     * @param provider
     */
    public static void providerAdded(BugzillaProvider provider) {
        IPersistenceAdapter adapter = getPersistenceAdapter(provider);
        if (adapter != null) {
            adapter.storeElement(provider, true /* flush to disk */);
        }
        provider.getEventHandler().addProviderListener(listener );
    }

    /**
     * The provider has been removed so purge any persisted state
     * @param provider
     */
    public static void providerRemoved(BugzillaProvider provider) {
        IPersistenceAdapter adapter = getPersistenceAdapter(provider);
        if (adapter != null) {
            adapter.purgeChild(null, provider, true /* flush to disk */);
        }
        provider.getEventHandler().removeProviderListener(listener);
    }

    /**
     * @param container
     */
    public static Preferences getPreferenceNode(BugzillaContainer container) {
        return AbstractPersistenceAdapter.getPreferences(container);
    }
    
    /*
     * Do not allow instantiation
     */
    private PersistenceManager() {
    }

}
