/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.tools;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.team.bugzilla.model.BugzillaContainer;
import org.eclipse.team.bugzilla.model.BugzillaModel;

/**
 * A wrapper for selections in a Bugzilla model. Various properties which are important for example for action
 * enablements can be queried. The findings are cached, such that for a series of queries for the same property,
 * the algorithms don't need to be executed twice.
 */
public class BugzillaSelection implements IStructuredSelection {
    
    private final IStructuredSelection fSelection;
    
    private BugzillaContainer fCommonParent;
    private Boolean fContainsReadOnlyElements;
    

    public BugzillaSelection() {
        this(StructuredSelection.EMPTY);
    }

    public BugzillaSelection(ISelection selection) {
        if (selection instanceof IStructuredSelection)
            fSelection= (IStructuredSelection)selection;
        else
            fSelection= StructuredSelection.EMPTY;
    }
    
    public BugzillaContainer getCommonParent() {
        if (fCommonParent == null) {
            fCommonParent= findCommonParent();
        }
        return fCommonParent;
    }
    
    public boolean containsReadOnlyElements() {
        if (fContainsReadOnlyElements == null)
            fContainsReadOnlyElements= findContainsReadOnlyElements() ? Boolean.TRUE : Boolean.FALSE;
        return fContainsReadOnlyElements.booleanValue();
    }
    
    public boolean isEmpty() {
        return fSelection.isEmpty();
    }
    
    public int size() {
        return fSelection.size();
    }
    
    public List toList() {
        return fSelection.toList();
    }
    
    public Object getFirstElement() {
        return fSelection.getFirstElement();
    }
    
    public BugzillaModel getOnlyElement() {
        return size() == 1 ? (BugzillaModel)getFirstElement() : null;
    }
    
    public BugzillaModel [] toBugzillaArray() {
        return (BugzillaModel [])toList().toArray(new BugzillaModel[size()]);
    }
    
    private boolean findContainsReadOnlyElements() {
        for (Iterator iter = fSelection.iterator(); iter.hasNext();) {
            if (((BugzillaModel)iter.next()).isReadOnly())
                return true;
        }
        return false;
    }

    private BugzillaContainer findCommonParent() {
        if (fSelection.isEmpty())
            return null;
        final Object parent= ((BugzillaModel)fSelection.getFirstElement()).getParent();
        if (!(parent instanceof BugzillaContainer))
            return null;
        for (final Iterator iter= fSelection.iterator(); iter.hasNext(); ) {
            if (((BugzillaModel)iter.next()).getParent() != parent)
                return null;
        }
        return (BugzillaContainer)parent;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.IStructuredSelection#iterator()
     */
    public Iterator iterator() {
        return fSelection.iterator();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.IStructuredSelection#toArray()
     */
    public Object[] toArray() {
        return fSelection.toArray();
    }
}
