package org.eclipse.team.bugzilla.view;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.team.bugzilla.model.BugzillaContainer;
import org.eclipse.team.bugzilla.model.BugzillaFolder;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.BugzillaQuery;
import org.eclipse.team.bugzilla.model.BugzillaReport;

/**
 * Class for serializing BugzillaModels to/from a byte array
 */
public class BugzillaTransfer extends ByteArrayTransfer {
    
    private static final String TYPE_NAME = "BugzillaModel-transfer-format";
    private static final int TYPEID = registerType(TYPE_NAME);
    
    /**
     * BugzillaModel serialization format is as follows:
     * (int) type of model
     * 
     * Folders:
     * (String) name of folder
     * (int) number of child models
     * (BugzillaModel) child 1
     * ... repeat for each child
     * 
     * Queries:
     * (String) Query URL
     * (int) number of child models
     * (BugzillaModel) child 1
     * ... repeat for each child
     * 
     * Reports:
     * (String) id
     */
    private static final int REPORT= 0;
    private static final int FOLDER= 1;
    private static final int QUERY= 2;
    
    
    private static BugzillaTransfer fInstance = new BugzillaTransfer();
    
    /**
     * Returns the singleton BugzillaModel transfer instance.
     */
    public static BugzillaTransfer getInstance() {
        return fInstance;
    }
    
    public static boolean isSupported(Object o) {
        return (o instanceof BugzillaReport) || (o instanceof BugzillaFolder) || (o instanceof BugzillaQuery);
    }
    
    /**
     * Avoid explicit instantiation
     */
    private BugzillaTransfer() {
    }
    
    protected BugzillaModel[] fromByteArray(byte[] bytes) {
        final DataInputStream in = new DataInputStream(new ByteArrayInputStream(bytes));
        
        try {
            /* read number of BugzillaModels */
            final int n = in.readInt();
            /* read BugzillaModels */
            final BugzillaModel [] BugzillaModels = new BugzillaModel[n];
            for (int i = 0; i < n; i++) {
                final BugzillaModel BugzillaModel = readBugzillaModel(null, in);
                if (BugzillaModel == null) {
                    return null;
                }
                BugzillaModels[i] = BugzillaModel;
            }
            return BugzillaModels;
        } catch (IOException e) {
            return null;
        }
    }
    
    /**
     * Reads and returns a single BugzillaModel from the given stream.
     */
    private BugzillaModel readBugzillaModel(BugzillaContainer parent, DataInputStream dataIn) throws IOException {
        
        final int type= dataIn.readInt();
        BugzillaModel model;
        
        switch (type) {
        case FOLDER:
        {
            final String name = dataIn.readUTF();
            final BugzillaFolder folder= new BugzillaFolder(parent, name);
            final int n = dataIn.readInt();
            final BugzillaModel [] children= new BugzillaModel[n];
            for (int i = 0; i < n; i++) {
                children[i]= readBugzillaModel(folder, dataIn);
            }
            folder.addChildren(children);
            model= folder;
            break;
        }
            
        case QUERY:
        {
            final String queryURL= dataIn.readUTF();
            final BugzillaQuery query= new BugzillaQuery(parent, queryURL);
            final int n = dataIn.readInt();
            for (int i = 0; i < n; i++) {
                query.addChildren(new BugzillaModel [] { readBugzillaModel(query, dataIn) });
            }
            model= query;
            break;
        }
            
        case REPORT:
        {
            final String id= dataIn.readUTF();
            model= new BugzillaReport(parent, id);
            break;
        }
        
        default:
            model= null;               
        }
        return model;
    }
    
    private byte [] toByteArray(BugzillaModel [] BugzillaModels) {
        
        final ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        final DataOutputStream out = new DataOutputStream(byteOut);
        
        try {
            out.writeInt(BugzillaModels.length);
            for (int i = 0; i < BugzillaModels.length; i++) {
                writeBugzillaModel(BugzillaModels[i], out);
            }
            out.close();
            return byteOut.toByteArray();
        } catch (IOException e) {
            return null;
        }
    }
    
    /**
     * Writes the given BugzillaModel to the stream.
     */
    private void writeBugzillaModel(BugzillaModel model, DataOutputStream dataOut) throws IOException {
        if (model instanceof BugzillaFolder) {
            dataOut.writeInt(FOLDER);
            final BugzillaFolder folder= (BugzillaFolder)model;
            dataOut.writeUTF(folder.getName());
            final BugzillaModel [] children= folder.getChildren();
            dataOut.writeInt(children.length);
            for (int i = 0; i < children.length; i++) {
                final BugzillaModel child = children[i];
                writeBugzillaModel(child, dataOut);
            }
            return;
        } 
        if (model instanceof BugzillaReport) {
            dataOut.writeInt(REPORT);
            final BugzillaReport report= (BugzillaReport)model;
            dataOut.writeUTF(report.getID());
            return;
        }
        
        if (model instanceof BugzillaQuery)	{
            dataOut.writeInt(QUERY);
            final BugzillaQuery query= (BugzillaQuery)model;
            dataOut.writeUTF(query.getQueryUrl().toUrlString());
            final BugzillaModel [] children= query.getChildren();
            dataOut.writeInt(children.length);
            for (int i = 0; i < children.length; i++) {
                final BugzillaModel child = children[i];
                writeBugzillaModel(child, dataOut);
            }
            return;
        }
        throw new IOException("No serialization for this type: " + model);
    }
    
    
    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.Transfer#getTypeIds()
     */
    protected int[] getTypeIds() {
        return new int[] { TYPEID };
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.Transfer#getTypeNames()
     */
    protected String[] getTypeNames() {
        return new String[] { TYPE_NAME };
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.Transfer#javaToNative(java.lang.Object, org.eclipse.swt.dnd.TransferData)
     */
    protected void javaToNative(Object object, TransferData transferData) {
        final byte [] bytes = toByteArray((BugzillaModel[])object);
        if (bytes != null)
            super.javaToNative(bytes, transferData);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.Transfer#nativeToJava(org.eclipse.swt.dnd.TransferData)
     */
    protected Object nativeToJava(TransferData transferData) {
        byte[] bytes = (byte[])super.nativeToJava(transferData);
        return fromByteArray(bytes);
    }
}