/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.view;

import java.util.Iterator;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.team.bugs.extension.IViewerProxy;
import org.eclipse.team.bugs.internal.views.dnd.BugsDragDropAdapter;
import org.eclipse.team.bugzilla.model.BugzillaContainer;
import org.eclipse.team.bugzilla.model.BugzillaModel;


class DropListener extends BugsDragDropAdapter implements DropTargetListener {
    
    private final DragListener fDragListener;
    private final IViewerProxy fProxy;
    
    private int fLastValidOperation;

    public DropListener(IViewerProxy proxy, DragListener dragListener) {
        fDragListener= dragListener;
        fProxy= proxy;
        final int operations= DND.DROP_MOVE | DND.DROP_LINK | DND.DROP_TARGET_MOVE | DND.DROP_DEFAULT | DND.DROP_COPY;
        final Transfer [] transfers= new Transfer [] { BugzillaTransfer.getInstance() };
        proxy.addDropSupport(operations, transfers, this);
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragEnter(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragEnter(DropTargetEvent event) {
        for (int i = 0; i < event.dataTypes.length; i++) {
            if (BugzillaTransfer.getInstance().isSupportedType(event.dataTypes[i])) {
                event.currentDataType= event.dataTypes[i];
                if ((event.detail & (DND.DROP_DEFAULT)) != 0)
                    event.detail= DND.DROP_MOVE;
            }
        }
        
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragLeave(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragLeave(DropTargetEvent event) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragOperationChanged(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragOperationChanged(DropTargetEvent event) {
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dragOver(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragOver(DropTargetEvent event) {
        final BugzillaContainer target= getValidatedTarget(event);
        
        if (event.detail != DND.DROP_NONE)
            fLastValidOperation= event.detail;
        
        if (target != null && target != fDragListener.getSourceContainer()) {
            event.detail= fLastValidOperation;
            event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
        } else {
            event.feedback= DND.FEEDBACK_SCROLL;
            event.detail= DND.DROP_NONE;
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#drop(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void drop(DropTargetEvent event) {
        if (!performDrop(event)) {
            event.detail= DND.DROP_NONE;
        }
    }
    
    private boolean performDrop(DropTargetEvent event) {
        if (!BugzillaTransfer.getInstance().isSupportedType(event.currentDataType))
            return false;
        
        final BugzillaContainer target= getValidatedTarget(event);

        if (target == null)
            return false;

        if (!(event.data instanceof BugzillaModel []))
        	return false;
        
        final BugzillaModel [] models= (BugzillaModel [])event.data;
        
        for (int i = 0; i < models.length; i++) {
            if (!target.contains(models[i].getID()))
                continue;
          
            /**
             * An element is already available - abort if the user says so.
             */
            if (!MessageDialog.openQuestion(fProxy.getPart().getSite().getShell(), "Overwrite?", "There are overlapping elements in the target location. Continue and overwrite all?")) 
                return false;
        }
        /**
         * Finally, perform the operation.
         */
        target.addChildren(models);
        return true;
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.dnd.DropTargetListener#dropAccept(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dropAccept(DropTargetEvent event) {
        if (BugzillaTransfer.getInstance().isSupportedType(event.currentDataType)) {
            if (getValidatedTarget(event) != null)
                return;
        }
        event.detail= DND.DROP_NONE;
    }
    
    private BugzillaContainer getValidatedTarget(DropTargetEvent event) {
        
        final Object element= getModelElement(event);
        
        if (!(element instanceof BugzillaContainer))
            return null;
        
        final BugzillaContainer target= (BugzillaContainer)element;
        
        if (target == fDragListener.getSourceContainer())
            return null;
        
        if (target.getProvider() != fDragListener.getSourceProvider())
            return null;
        
        if (!target.canModifyChildren()) 
            return null;
        
        final IStructuredSelection selection= fDragListener.getDraggedSelection();
        for (Iterator iter = selection.iterator(); iter.hasNext();) {
            if (target.hasParent(iter.next()))
                return null;
        }
        return target;
    }
}