/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.view;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.team.bugs.model.IGenericModel;
import org.eclipse.team.bugs.model.IProviderListener;
import org.eclipse.team.bugzilla.browser.BugzillaBrowser;
import org.eclipse.team.bugzilla.browser.IWebContentAdapter;
import org.eclipse.team.bugzilla.model.BugzillaModel;
import org.eclipse.team.bugzilla.model.BugzillaProvider;
import org.eclipse.team.bugzilla.tools.BugzillaSelection;
import org.eclipse.team.bugzilla.tools.IBugzillaSelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.PageBook;

public class TextPane implements IBugzillaSelectionListener, IProviderListener, IOpenListener {
    
    private StyledText fStyledText;
    //private Control fEmptyControl;
    private PageBook fPageBook;
    private SashForm sashForm = null;
    private int[] userWeights = {2, 1};
    
    private BugzillaSelection fSelection;
    
    private final DelayedRefreshPresenter fDelayedRefresh;
    private BugzillaViewExtension fExtension;
    
    public TextPane(BugzillaViewExtension extension, Composite parent) {
        fExtension= extension;
        fDelayedRefresh= new DelayedRefreshPresenter();
        
        fPageBook= new PageBook(parent, SWT.NONE);
        //fEmptyControl= new Composite(fPageBook, SWT.NONE);
        fStyledText= new StyledText(fPageBook, SWT.H_SCROLL | SWT.V_SCROLL | SWT.READ_ONLY);
        fSelection= new BugzillaSelection(StructuredSelection.EMPTY);
        
        final ActionManager manager= fExtension.getActionManager();
        manager.createTextContextMenu(fStyledText);
        
        if (parent.getParent() instanceof SashForm) {
        	sashForm = (SashForm)parent.getParent();
        	hideView();
        }
    }

    public void selectionChanged(BugzillaSelection selection) {
        fSelection= selection;

        final BugzillaModel element= fSelection.getOnlyElement();
        fDelayedRefresh.refresh(element);
        
        if (element != null) {
            present(element);
        }
        else {
        	hideView();
        }
    }


    public void elementsAdded(IGenericModel parent, IGenericModel[] children) {
    }

    public void elementsChanged(IGenericModel[] elements) {
        /**
         * We only update the styled text if the model has just refreshed
         * and is currently selected.
         */
        final BugzillaModel selected= fSelection.getOnlyElement();
        if (selected == null)
            return;
        
        for (int i = 0; i < elements.length; i++) {
            if (selected == elements[i] && !selected.isRefreshing()) {
                present(selected);
            }
        }
    }

    public void elementsRemoved(IGenericModel parent, IGenericModel[] children) {
    }
    
    private void present(BugzillaModel element) {
    	
    	final IPresentableAdapter adapter= getPresentableAdapter(element);
        if (adapter == null) {
        	hideView();
        	return;
        }
                
        adapter.present(element, fStyledText);
        //final boolean hasText= fStyledText.getText().trim().length() != 0;
        fPageBook.showPage(fStyledText);
        
        if (sashForm.getWeights()[1] == 0) {
        	showView();
        }
    }
    
    private void showView() {
    	if (sashForm == null) {
    		return;
    	}
    	sashForm.setWeights(userWeights);
    }
    
    private void hideView() {
    	if (sashForm == null) {
    		return;
    	}
    	if  ((sashForm.getWeights())[1] != 0) {
    		userWeights = sashForm.getWeights();
    	}
    	sashForm.setWeights(new int[] {1, 0});
    }
    
    private IPresentableAdapter getPresentableAdapter(Object o) {
        if (o instanceof IAdaptable) {
            return (IPresentableAdapter)((IAdaptable)o).getAdapter(IPresentableAdapter.class);
        }
        return null;
    }

    public void open(OpenEvent event) {
        if (!(event.getSelection() instanceof IStructuredSelection))
            return;
        
        final IStructuredSelection selection= (IStructuredSelection)event.getSelection();
        
        if (!(selection.getFirstElement() instanceof BugzillaModel))
            return;
        
        final BugzillaModel model= (BugzillaModel)selection.getFirstElement();
        
        final IWebContentAdapter adapter= (IWebContentAdapter)model.getAdapter(IWebContentAdapter.class);
        if (adapter == null)
            return;
        
        final LocationListener listener= (BugzillaProvider)model.getProvider();
        
        final String url= adapter.getURL(model);
        if (url != null) {
            final IWorkbenchPart part= PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart();
            BugzillaBrowser.open(part, listener, url.replaceAll("format=rdf", ""));
        }
        
    }

//    public String getSelection() {
//        return fStyledText.getSelectionText();
//    }
//
//    public boolean hasFocus() {
//        return fStyledText.isFocusControl();
//    }
    
    public StyledText getStyledText() {
        return fStyledText;
    }
    
    public Control getControl() {
        return fPageBook;
    }
}