/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.wizards;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.team.bugs.tools.SWTUtils;
import org.eclipse.team.bugzilla.datatypes.BugzillaConfiguration;

/**
 * 
 */
public class BugzillaWizardPage extends StatusWizardPage {
    
    private final class NameField implements ModifyListener {
        
        private final Text fNameText;
        private String fName;
        
        public NameField(Composite composite) {
            final Label label= new Label(composite, SWT.WRAP);
            label.setLayoutData(SWTUtils.createHFillGridData());
            label.setText("Name:");
            fNameText= new Text(composite, SWT.SINGLE | SWT.BORDER);
            fNameText.setLayoutData(SWTUtils.createHFillGridData());
            fNameText.addModifyListener(this);
            fName= null;
            fNameText.setFocus();
            setBlocker(this, true);
        }
        
        public String getName() {
            return fName;
        }
		
		public void setName(String n) {
			fName = n;
			fNameText.setText(n);
			modifyText(null);
		}
        
        public void modifyText(ModifyEvent e) {
            final boolean valid= fNameText.getText().trim().length() != 0;
            fName= valid ? fNameText.getText() : null;
            setErrorMessage(this, valid ? null : "Please enter a name.");
        }
    }
    
    private final class LocationField implements ModifyListener {
        
        private final Combo fLocationText;
        private String fLocation;
        
        public LocationField(Composite composite) {
            final Label locationLabel= new Label(composite, SWT.WRAP);
            locationLabel.setLayoutData(SWTUtils.createHFillGridData());
            locationLabel.setText("Location (URL):");
            
            fLocationText = new Combo(composite, SWT.SINGLE | SWT.BORDER);
            fLocationText.setLayoutData(SWTUtils.createHFillGridData());
            fLocationText.addModifyListener(this);
            
			Iterator iter = preSets.keySet().iterator();
			while (iter.hasNext()) {
				fLocationText.add((String)iter.next());
			}
           
			fLocationText.addSelectionListener(new SelectionListener() {

				public void widgetSelected(SelectionEvent e) {
					String name = fNameField.getName();
					if ((name == null || name.trim().equals("")) || preSets.containsValue(name)) {
						fNameField.setName((String)preSets.get(fLocationText.getText()));
					}
				}

				public void widgetDefaultSelected(SelectionEvent e) {
					
				}
				
			});
			
            setBlocker(this, true);
        }
        
        public void modifyText(ModifyEvent e) {
            if (validateLocation(fLocationText.getText())) {
                fLocation= fLocationText.getText();
                setErrorMessage(this, null);
            } else {
                fLocation= null;
                setErrorMessage(this, "Please enter a valid URL.");
            }
        }
        
        public String getLocation() {
            return fLocation;
        }

        private boolean validateLocation(String location) {
            final URL url;
            try {
                url= new URL(location);
            } catch (MalformedURLException e) {
                return false;
            }
            if (!url.getFile().equals(url.getPath()))
                return false;
            
            if (!"http".equals(url.getProtocol()) && !"https".equals(url.getProtocol())) //$NON-NLS-1$
                return false;
            
            if (url.getHost().trim().length() == 0)
                return false;
            
            return true;
        }

		public void setLocation(String baseURL) {
			fLocation = baseURL;
		}
    }
    
    private NameField fNameField;
    private LocationField fLocationField;
	private static HashMap preSets;
	
	static {
		preSets = new HashMap(6);
		preSets.put("https://bugs.eclipse.org/bugs", "Eclipse");
		preSets.put("https://bugzilla.redhat.com/bugzilla", "Red Hat");
		preSets.put("https://bugzilla.mozilla.org", "Mozilla"); /* 2.19 version not supported */
		//preSets.put("https://bugs.freedesktop.org", "Freedesktop.org"); /* security validation error ? */
		preSets.put("http://bugzilla.gnome.org", "Gnome");
		preSets.put("http://bugzilla.kernel.org", "Kernel");
		preSets.put("http://issues.apache.org/bugzilla/", "Apache");
		preSets.put("http://gcc.gnu.org/bugzilla/", "GCC");
		//preSets.put("http://bugs.kde.org", "KDE"); /* non-standard bugzilla */
        //preSets.put("http://bugzilla.ubuntu.com/", "Ubuntu");
	}
    
	protected BugzillaConfiguration bugConf = null;
	
    protected BugzillaWizardPage() {
        super("Add Bugzilla");
        setTitle("Add a Bugzilla bug tracking system.");
        setDescription("Please specify a name and a valid location.");
    }
    
    public BugzillaWizardPage(BugzillaConfiguration c) {
    	super("Add Bugzilla");
        setTitle("Edit Bugzilla bug tracking system.");
        setDescription("Please specify a name and a valid location.");
        bugConf = c;
    }

    public void createControl(Composite parent) {
        initializeDialogUnits(parent);
        
        final Composite composite= new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(1, false));
        
        fNameField= new NameField(composite);
        fLocationField= new LocationField(composite);
        
        setControl(composite);
        setPageComplete(false);
        
        if (bugConf != null) {
        	fNameField.setName(bugConf.getName());
        	fLocationField.setLocation(bugConf.getBaseURL());
        	setPageComplete(true);
        }
    }
    
    public BugzillaConfiguration getConfiguration() {
        final String name= fNameField.getName();
        final String url= fLocationField.getLocation();
        if (name == null || url == null) 
            return null;
        return new BugzillaConfiguration(name, url);
    }
}   

