/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.team.bugzilla.xml;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.team.bugs.extension.BugsException;
import org.eclipse.team.bugzilla.BugzillaException;
import org.eclipse.team.bugzilla.BugzillaPlugin;
import org.eclipse.team.bugzilla.datatypes.AttachmentDescription;
import org.eclipse.team.bugzilla.datatypes.CommentData;
import org.eclipse.team.bugzilla.datatypes.ReportData;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


public class ReportHandler_2_16 extends ReportHandler implements Tokens {
    
	protected final ArrayList fAttachments;
    
	protected CommentData fComment;
	protected AttachmentDescription fAttachment;
    
    protected ReportData fReport;
    
    public ReportHandler_2_16() {
        super();
        fAttachments= new ArrayList();
        fReport= null;
        fComment= null;
        fAttachment= null;
    }
    
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {

        final int token= LookupTable.get(qName);
        
        switch (token) {

        case BUGZILLA:
            final String value= attributes.getValue("version"); //$NON-NLS-1$
            if (value == null || !value.startsWith("2.16.")) //$NON-NLS-1$
                throw new SAXException("The actual version doesn't match the expected version!"); //$NON-NLS-1$
            fReport= new ReportData();
            fReport.set(ReportData.BUGZILLAVERSION, value);
            break;

        case LONG_DESC:
            fComment= new CommentData();
            break;

        case ATTACHMENT:
            fAttachment= new AttachmentDescription();
            break;
        }
    }
    
    public void endElement(String uri, String localName, String qName) throws SAXException {
        
        final String name= "".equals(localName) ? qName : localName;  //$NON-NLS-1$
        
        final int token= LookupTable.get(name);
        
        switch (token) {
        
        case BUG_ID:
            fReport.set(ReportData.ID, getPCDATA());
            break;
            
        case BUG_STATUS:
            fReport.set(ReportData.STATUS, getPCDATA());
            break;
            
        case PRODUCT:
            fReport.set(ReportData.PRODUCT, getPCDATA());
            break;
            
        case PRIORITY:
            fReport.set(ReportData.PRIORITY, getPCDATA());
            break;
            
        case VERSION:
            fReport.set(ReportData.VERSION, getPCDATA());
            break;
            
        case REP_PLATFORM:
            fReport.set(ReportData.PLATFORM, getPCDATA());
            break;
            
        case ASSIGNED_TO:
            fReport.set(ReportData.ASSIGNEDTO, getPCDATA());
            break;
            
        case DELTA_TS:
            fReport.set(ReportData.DELTATIMESTAMP, getPCDATA());
            break;
            
        case COMPONENT:
            fReport.set(ReportData.COMPONENT, getPCDATA());
            break;
            
        case REPORTER:
            flushBuffer();// implied in first comment
            break;
            
        case TARGET_MILESTONE:
            fReport.set(ReportData.TARGETMILESTONE, getPCDATA());
            break;
            
        case BUG_SEVERITY:
            fReport.set(ReportData.SEVERITY, getPCDATA());
            break;
            
        case CREATION_TS:
            flushBuffer(); // implied in first comment
            break;
            
        case QA_CONTACT:
            fReport.set(ReportData.QACONTACT, getPCDATA());
            break;
            
        case OP_SYS:
            fReport.set(ReportData.OPERATINGSYSTEM, getPCDATA());
            break;
            
        case RESOLUTION:
            fReport.set(ReportData.RESOLUTION, getPCDATA());
            break;
            
        case BUG_FILE_LOC:
            fReport.set(ReportData.BUGFILELOCATION, getPCDATA());
            break;
            
        case SHORT_DESC:
            fReport.set(ReportData.TITLE, getPCDATA());
            break;
            
        case KEYWORDS:
            fReport.keywords.add(getPCDATA());
            break;
            
        case STATUS_WHITEBOARD:
            fReport.set(ReportData.STATUSWHITEBOARD, getPCDATA());
            break;
            
        case DEPENDSON:
            fReport.dependsOn.add(getPCDATA());
            break;
            
        case BLOCKS:
            fReport.blocks.add(getPCDATA());
            break;
            
        case CC:
            fReport.ccs.add(getPCDATA());
            break;
            
        /**
         * Handle comments. 
         */
        case LONG_DESC:
            fReport.comments.add(fComment);
            fComment = null;
            break;
        case WHO:
            fComment.set(CommentData.AUTHOR, getPCDATA());
            break;
            
        case BUG_WHEN:
            fComment.set(CommentData.DATE, getDate());
            break;
            
        case THETEXT:
            fComment.set(CommentData.TEXT, getPCDATA());
            break;
            
        /**
         * Handle attachments. 
         */
        case ATTACHMENT:
            fAttachments.add(fAttachment);
            fAttachment= null;
            break;
            
        case ATTACHID:
            fAttachment.set(AttachmentDescription.ID, getPCDATA());
            break;
            
        case DATE:
            fAttachment.set(AttachmentDescription.DATE, getAttachmentDate());
            break;
            
        case DESC:
            fAttachment.set(AttachmentDescription.DESCRIPTION, getPCDATA());
            break;
        case CTYPE:
            fAttachment.set(AttachmentDescription.CONTENTTYPE, getPCDATA());
            break;
         
        case BUGZILLA:
            /**
             * Add attachments to the respective comments.
             */
            for (final Iterator attachmentsIter = fAttachments.iterator(); attachmentsIter.hasNext();) {

                final AttachmentDescription attachment = (AttachmentDescription)attachmentsIter.next();
                final String pattern= "(id=" + attachment.getString(AttachmentDescription.ID).trim() + ")";  //$NON-NLS-1$//$NON-NLS-2$

                attachment.set(AttachmentDescription.OBSOLETE, false);
                CommentData associatedComment= null;
                
                for (final Iterator commentsIter= fReport.comments.iterator(); commentsIter.hasNext(); ) {

                    final CommentData comment= (CommentData)commentsIter.next();
                    
                    if (comment.getString(CommentData.TEXT).indexOf(pattern) >= 0) { 
                        associatedComment= comment;
                        final boolean obsolete= !comment.getDate(CommentData.DATE).equals(attachment.getDate(AttachmentDescription.DATE));
                        attachment.set(AttachmentDescription.OBSOLETE, obsolete);
                        break;
                    }
                }
                if (associatedComment == null) {
                    final IStatus status= new Status(IStatus.ERROR, 
                            BugzillaPlugin.ID, 
                            BugzillaException.UNEXPECTED_CONDITION, 
                            "Attachment " + attachment.getString(AttachmentDescription.ID) + " is not associated to a comment!", 
                            null);
                    BugzillaPlugin.log(new BugsException(status));
                } else {
                    associatedComment.attachmentDesc= attachment;
                }
             }
        }
    }
        
    public void error(SAXParseException e) throws SAXException {
        throw e;
    }
        
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException {
        if (!systemId.endsWith("bugzilla.dtd"))  //$NON-NLS-1$
            return null;
        
        return null;
//        try{
//            return new InputSource(new FileReader("")); //$NON-NLS-1$
//        } catch (FileNotFoundException e) {
//            return null;
//        }
    }
    
    public ReportData getReportData() {
        return fReport;
    }
}